/* clebsg.c */

/*=============================================================*/
/*  ADAPTED FROM HARWELL LIBRARY BY T. LINDELOF AND F. JAMES   */
/*  08/01/74 LAST UPDATE OF HARWELL LIBRARY                    */
/*=============================================================*/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <stdlib.h>
#include <math.h>


#define IARITHIF(x)  (x > 0 ? 1 : x < 0 ? -1 : 0)

struct t_fgercm 
{
  int ierr, ierct;
}fgercm;

typedef double doublereal;
typedef long int integer;

double sign(double a, double b);
double clebsg(double a, double b, double c, double xx, double yy, double zz, double gg, double hh, double pp, int mode);


double clebsg(double a, double b, double c, double xx, double yy, double zz, double gg, double hh, double pp,int mode)
{

int i, i_,iay[4],ij, ijpar, iyy, iz, jq, js, jspar,
	 k, k1, k2, k3, k4, k5, k6, ka, kb, kc, key, keyrac, keytri, kk1,
	 kk2, kk3, kk4, kk5, kk6, kk7, kk8, kk9, kup, m1, m10, m2, m3,
	 m4, m5, m6, m7, m8, m9, ma, mb, mc, md, mm1, mm2, mm3, mm4, mm5,
	 n4, n5, n5par;
double anine,ay[4],clebsg_v, clebsh, factor, ra,
	 racah, rb, x, y, z;

static int  ix,iy,j[101];
static double h[101];
static int jjj = 0;
static double signz = 1.;


	/*         ADAPTED FROM HARWELL LIBRARY BY T. LINDELOF AND F. JAMES
	 *         08/01/74 LAST UPDATE OF HARWELL LIBRARY
	 *
	 *     WIGN3J- WIGNER 3-J SYMBOL
	 *     CLEBSG- CLEBSCH-GORDAN COEFFICIENT
	 *     WIGN6J- WIGNER 6-J SYMBOL
	 *     RACAHC- RACAH COEFFICIENT
	 *     JAHNUF- U-FUNCTION (JAHN)
	 *     WIGN9J- WIGNER 9-J SYMBOL
	 */

#define INTPTF(q)       (int)((q) + (q) + sign( .10e0, (q) ))
#define IPARF(i)        (int)(4*((i)/4) - (i) + 1)

	if(mode==1) goto wign3j;
	else
	 if(mode==2) goto wign6j;
	 else
	  {
	   printf("ERROR!!!");
	   exit(1);
	  }

wign3j:
        if(xx + yy +zz != 0.0) return(0.0);  /* speed up of 3j symbols */
	fgercm.ierr = 0;
	/*     CALL NOARG(NARG)
	 *     IF(NARG.EQ.3) GOTO 2 */
	signz = -1.;
	key = 1;
	goto L_1;
	/*   2 KEY=3
	 *     GOTO 1
	 */


wign6j:
	key = 11;
	fgercm.ierr = 0;
	goto L_100;




L_1:
	k1 = INTPTF( a );
	k2 = INTPTF( b );
	k3 = INTPTF( c );
	if( key >= 3 )
		goto L_100;
	k4 = INTPTF( xx );
	k5 = INTPTF( yy );
	k6 = INTPTF( zz*signz );

L_100:
	if( jjj != 0 )
		goto L_500;
	jjj = 1;
	fgercm.ierct = 0;
	h[0] = 1.0;
	j[0] = 0;
	x = 0.;
	for( i = 2; i <= 101; i++ ){
		i_ = i - 1;
		x = x + 1.0;
		h[i_] = h[i_ - 1]*x;
		j[i_] = j[i_ - 1];
L_200:
		if( h[i_] < 10.0 )
			goto L_400;
		h[i_] = 0.01*h[i_];
		j[i_] = j[i_] + 2;
		goto L_200;
L_400:
		;
		}

L_500:
	if( key < -5 )
		goto L_750;
        if( key >= 3 )
                goto L_320;
        if( (k4 + k5 - k6) != 0 )
                goto L_710;
        m1 = k1 + k2 - k3;
	m2 = k2 + k3 - k1;
        m3 = k3 + k1 - k2;
        m4 = k1 + k4;
	m5 = k1 - k4;
        m6 = k2 + k5;
        m7 = k2 - k5;
        m8 = k3 + k6;
        m9 = k3 - k6;
        m10 = k1 + k2 + k3 + 2;

        if( m1 < 0 )
                goto L_710;
        if( m2 < 0 )
		goto L_710;
	if( m3 < 0 )
                goto L_710;
        if( m4 < 0 )
                goto L_710;
        if( m5 < 0 )
                goto L_710;
	if( m6 < 0 )
                goto L_710;
        if( m7 < 0 )
		goto L_710;
        if( m8 < 0 )
                goto L_710;
        if( m9 < 0 )
                goto L_710;
        if( (m4 - (m4/2) - (m4/2)) != 0 )
                goto L_710;
        if( (m6 - (m6/2) - (m6/2)) != 0 )
                goto L_710;
        if( (m8 - (m8/2) - (m8/2)) != 0 )
		goto L_710;
	if( (m10 - (m10/2) - (m10/2)) != 0 )
                goto L_710;

        y = k3 + 1;
        m1 = m1/2 + 1;
        m2 = m2/2 + 1;
	m3 = m3/2 + 1;
        m4 = m4/2 + 1;
        m5 = m5/2 + 1;
	m6 = m6/2 + 1;
        m7 = m7/2 + 1;
        m8 = m8/2 + 1;
        m9 = m9/2 + 1;
        m10 = m10/2 + 1;

        y = sqrt( y*h[m1 - 1]*h[m2 - 1]*h[m3 - 1]*h[m4 - 1]*h[m5 - 1]*
         h[m6 - 1]*h[m7 - 1]*h[m8 - 1]*h[m9 - 1]/h[m10 - 1] );
        iy = j[m1 - 1] + j[m2 - 1] + j[m3 - 1] + j[m4 - 1] + j[m5 - 1] +
         j[m6 - 1] + j[m7 - 1] + j[m8 - 1] + j[m9 - 1] - j[m10 - 1];
	iy=(int)((float)(iy)/2.0);

        n4 = m1;
        if( n4 > m5 )
                n4 = m5;
        if( n4 > m6 )
                n4 = m6;
	n4 = n4 - 1;
        m2 = k2 - k3 - k4;
        m3 = k1 + k5 - k3;
	n5 = 0;
        if( n5 < m2 )
                n5 = m2;
        if( n5 < m3 )
                n5 = m3;
        n5par = IPARF( n5 );
        n5 = n5/2;
        z = 0.0;
        goto L_610;

L_700:
	 mm1 = m1 - n5;
         mm2 = m5 - n5;
         mm3 = m6 - n5;
         mm4 = n5 - (m2/2) + 1;
         mm5 = n5 - (m3/2) + 1;

	   x = 1./(h[mm1 - 1]*h[mm2 - 1]*h[mm3 - 1]*h[mm4 - 1]*h[mm5 - 1]*
            h[n5]);

	 ix = -j[mm1 - 1] - j[mm2 - 1] - j[mm3 - 1] - j[mm4 - 1] - j[mm5 - 1] -
          j[n5];

L_800:
        switch( IARITHIF(ix + iy) ){
                case -1: goto L_900;
                case  0: goto L_210;
                case  1: goto L_110;
                }
L_900:
	x = 0.1*x;
	ix = ix + 1;
        goto L_800;
L_110:
        x = 10.0*x;
        ix = ix - 1;
        goto L_800;

L_210:
        if( n5par < 0 )
		x = -x;
        z = z + x;
        n5par = -n5par;
        n5 = n5 + 1;

L_610:
        switch( IARITHIF(n5 - n4) ){
                case -1: goto L_700;
                case  0: goto L_700;
                case  1: goto L_810;
		}

L_710:
        clebsh = 0.0;
        fgercm.ierr = 1;
        fgercm.ierct = fgercm.ierct + 1;
        goto L_220;

L_810:
        clebsh = z*y;
	switch( key ){
                case 1: goto L_120;
                case 2: goto L_220;
                }

L_220:
        clebsg_v = clebsh;
        return( clebsg_v );

L_120:
	js = k1 - k2 + k6;
	if( js < 0 )
                js = -js;
        jspar = IPARF( js );
	clebsg_v = jspar*clebsh/sqrt( k3 + 1.0 );
	signz = 1.;
        return( clebsg_v );

L_320:
        if( key >= 10 )
		goto L_130;
        key = key - 2;
        if( (k1 - (k1/2) - (k1/2)) != 0 )
                goto L_420;
        if( (k2 - (k2/2) - (k2/2)) != 0 )
                goto L_420;
        if( (k3 - (k3/2) - (k3/2)) != 0 )
                goto L_420;
        ij = k1 + k2 + k3;
        ijpar = IPARF( ij );
	if( ijpar <= 0 )
		goto L_420;
        m1 = ij - k1 - k1;
        m2 = ij - k2 - k2;
        m3 = ij - k3 - k3;
        m4 = ij + 2;
        if( m1 < 0 )
		goto L_420;
        if( m2 < 0 )
                goto L_420;
	if( m3 < 0 )
                goto L_420;
        m1 = m1/2 + 1;
        m2 = m2/2 + 1;
        m3 = m3/2 + 1;
        m4 = ij/2 + 2;
        y = sqrt( h[m1 - 1]*h[m2 - 1]*h[m3 - 1]/h[m4 - 1] );
        iy = (j[m1 - 1] + j[m2 - 1] + j[m3 - 1] - j[m4 - 1])/2;
        ij = ij/2;
        ijpar = IPARF( ij );
	ij = ij/2 + 1;
	m1 = m1/2 + 1;
        m2 = m2/2 + 1;
        m3 = m3/2 + 1;
        z = h[ij - 1]/(h[m1 - 1]*h[m2 - 1]*h[m3 - 1]);
        iz = j[ij - 1] - j[m1 - 1] - j[m2 - 1] - j[m3 - 1];
        iz = iz + iy;
	clebsh = ijpar*y*z*pow(10.0, iz);
        switch( key ){
                case 1: goto L_220;
		case 2: goto L_720;
                }

L_720:
        jq = k2 - k1;
        if( jq < 0 )
                jq = -jq;
        ijpar = IPARF( jq );
        clebsg_v = clebsh*ijpar*sqrt( k3 + 1.0 );
        return( clebsg_v );

L_420:
        clebsh = 0.0;
        fgercm.ierr = 1;
        fgercm.ierct = fgercm.ierct + 1;
        switch( key ){
                case 1: goto L_220;
		case 2: goto L_720;
                }

L_130:
        if( key == 11 )
                goto L_450;
        if( key > 19 )
                goto L_750;
	k1 = INTPTF( a );
	k2 = INTPTF( b );
	k3 = INTPTF( yy );
	k4 = INTPTF( xx );
	k5 = INTPTF( c );
	k6 = INTPTF( zz );

L_750:
        ka = k1;
        kb = k2;
        kc = k3;
        keytri = 1;
	goto L_630;

L_230:
	ka = k4;
        kb = k5;
        keytri = 2;
        goto L_630;

L_330:
        kb = k2;
        kc = k6;
        keytri = 3;
        goto L_630;

L_430:
        ka = k1;
        kb = k5;
        keytri = 4;
        goto L_630;

L_530:
	y = ay[0]*ay[1]*ay[2]*ay[3];
        iyy = iay[0] + iay[1] + iay[2] + iay[3];
	m1 = (k1 + k2 + k4 + k5)/2 + 2;
        m2 = (k1 + k2 - k3)/2 + 1;
        m3 = (k4 + k5 - k3)/2 + 1;
        m4 = (k1 + k5 - k6)/2 + 1;
        m5 = (k2 + k4 - k6)/2 + 1;
        m6 = k1 + k4 - k3 - k6;
        m7 = k2 + k5 - k3 - k6;

        n4 = m1;
        if( n4 > m2 )
		n4 = m2;
	if( n4 > m3 )
                n4 = m3;
        if( n4 > m4 )
                n4 = m4;
        if( n4 > m5 )
                n4 = m5;
	n4 = n4 - 1;
        n5 = 0;
        if( n5 < m6 )
		n5 = m6;
        if( n5 < m7 )
                n5 = m7;
        n5par = IPARF( n5 );
        n5 = n5/2;
        m6 = m6/2 - 1;
        m7 = m7/2 - 1;
        z = 0.0;
        goto L_730;

L_140:
	x = h[m1 - n5 - 1]/(h[n5]*h[m2 - n5 - 1]*h[m3 - n5 - 1]*h[m4 - n5 - 1]*
         h[m5 - n5 - 1]*h[n5 - m6 - 1]*h[n5 - m7 - 1]);
        ix = j[m1 - n5 - 1] - j[n5] - j[m2 - n5 - 1] - j[m3 - n5 - 1] -
         j[m4 - n5 - 1] - j[m5 - n5 - 1] - j[n5 - m6 - 1] - j[n5 - m7 - 1];
L_240:
        switch( IARITHIF(ix + iyy) ){
		case -1: goto L_340;
                case  0: goto L_440;
                case  1: goto L_540;
		}
L_340:
        x = 0.1*x;
        ix = ix + 1;
        goto L_240;
L_540:
        x = 10.0*x;
        ix = ix - 1;
        goto L_240;
L_440:
	if( n5par < 0 )
		x = -x;
        z = z + x;
        n5par = -n5par;
        n5 = n5 + 1;

L_730:
	if( n5 <= n4 )
                goto L_140;

	racah = z*y;
L_840:
        if( key < -5 )
                goto L_160;
        key = key - 10;
        switch( key ){
                case 1: goto L_150;
                case 2: goto L_250;
                case 3: goto L_350;
                }

L_830:
        racah = 0.0;
        fgercm.ierr = 1;
        fgercm.ierct = fgercm.ierct + 1;
        goto L_840;

L_150:
        ijpar = IPARF( k1 + k2 + k4 + k5 );
        if( ijpar < 0 )
		racah = -racah;
L_250:
        clebsg_v = racah;
        return( clebsg_v );

L_350:
        factor = sqrt( (k3 + 1.0)*(k6 + 1) );
        clebsg_v = factor*racah;
        return( clebsg_v );
L_450:
	k1 = INTPTF( a );
	k2 = INTPTF( b );
	k3 = INTPTF( c );
	k4 = INTPTF( xx );
	k5 = INTPTF( yy );
	k6 = INTPTF( zz );
        goto L_750;

        /*     TRIANGLE FUNCTION
         */
L_630:
        ma = ka + kb - kc;
        mb = ka - kb + kc;
        mc = -ka + kb + kc;
        md = ka + kb + kc + 2;
        if( ma < 0 )
                goto L_830;
        if( mb < 0 )
                goto L_830;
        if( mc < 0 )
		goto L_830;
	if( (md - (md/2) - (md/2)) != 0 )
                goto L_830;
        ma = ma/2 + 1;
        mb = mb/2 + 1;
        mc = mc/2 + 1;
        md = md/2 + 1;
	ay[keytri - 1] = sqrt( h[ma - 1]*h[mb - 1]*h[mc - 1]/h[md - 1] );


	iay[keytri - 1] = j[ma - 1] + j[mb - 1] + j[mc - 1] - j[md - 1];

        iay[keytri - 1]=(int)((float)(iay[keytri - 1])/2.0);


        switch( keytri ){
                case 1: goto L_230;
                case 2: goto L_330;
                case 3: goto L_430;
                case 4: goto L_530;
		}


        key = -10;
        fgercm.ierr = 0;

	kk1 = INTPTF( a );
	kk2 = INTPTF( b );
	kk3 = INTPTF( c );
	kk4 = INTPTF( xx );
	kk5 = INTPTF( yy );
	kk6 = INTPTF( zz );
	kk7 = INTPTF( gg );
	kk8 = INTPTF( hh );
	kk9 = INTPTF( pp );

	kup = kk1 + kk9;
	m1 = kk4 + kk8;
	m2 = kk2 + kk6;
	if( kup > m1 )
		kup = m1;
	if( kup > m2 )
		kup = m2;

	k = kk1 - kk9;
	if( k < 0 )
                k = -k;
        m1 = kk4 - kk8;
        if( m1 < 0 )
                m1 = -m1;
	m2 = kk2 - kk6;
        if( m2 < 0 )
                m2 = -m2;
        if( k < m1 )
                k = m1;
        if( k < m2 )
                k = m2;

        anine = 0.0;

L_660:
	if( k > kup )
                goto L_260;
        k1 = kk1;
        k2 = kk4;
	k3 = kk7;
        k4 = kk8;
        k5 = kk9;
        k6 = k;
        keyrac = 1;
	goto L_100;

L_160:
        switch( keyrac ){
                case 1: goto L_360;
                case 2: goto L_460;
                case 3: goto L_560;
                }

L_360:
	ra = racah;
	k1 = kk2;
        k2 = kk8;
        k3 = kk5;
        k4 = kk4;
	k5 = kk6;
        keyrac = 2;
        goto L_750;

L_460:
	rb = racah;
        k1 = kk9;
        k2 = kk6;
        k3 = kk3;
        k4 = kk2;
        k5 = kk1;
        keyrac = 3;
        goto L_750;

L_560:
	anine = anine + ra*rb*racah*(k + 1);
	k = k + 2;
        goto L_660;

L_260:
	clebsg_v = anine;
        return( clebsg_v );
#undef  IPARF
#undef  INTPTF
} /* end of function */


double sign(double a, double b)
{
  
  if(b>=0.0) 
    return(fabs(a));
  else 
    return(-fabs(a));
}






double tj(int l1, int l2, int l3, 
	  int ml1, int ml2, int ml3 )
{

  return( clebsg((double)l1, (double)l2, (double)l3, 
		 (double)ml1, (double)ml2, (double)ml3, 0.0, 0.0, 0.0, 1));


}




double sj(int l1, int l2, int l3, 
	  int l4, int l5, int l6)
{

  return( clebsg((double)l1, (double)l2, (double)l3, 
		 (double)l4, (double)l5, (double)l6, 0.0, 0.0, 0.0, 2));


}


double red(int l1, int l2, int l3)
{
  double fac;

  fac = pow(-1.0,l1) * sqrt( (2.0*l1+1.0) * (2.0*l3+1.0) );

  return( fac * tj(l1,l2,l3,0.0,0.0,0.0)  );


}

double nj(int l1, int l2, int l3, 
	  int l4, int l5, int l6,
	  int l7, int l8, int l9)
{
  int k, kmax;
  double sum=0.0, sj1,sj2,sj3;

  kmax=0;
  
  if(l1>kmax)
    kmax=l1;
  if(l2>kmax)
    kmax=l2;
  if(l3>kmax)
    kmax=l3;
  if(l4>kmax)
    kmax=l4;
  if(l5>kmax)
    kmax=l5;
  if(l6>kmax)
    kmax=l6;
  if(l7>kmax)
    kmax=l7;
  if(l8>kmax)
    kmax=l8;
  if(l9>kmax)
    kmax=l9;
   
  for(k=0;k<=2*kmax;k++)
    {
      sj1=sj(l1,l4,l7,l8,l9,k);
      sj2=sj(l2,l5,l8,l4,k,l6);
      sj3=sj(l3,l6,l9,k,l1,l2);

      sum = sum + pow(-1.0,(double)(2*k)) * (double)(2*k+1)
	* sj1*sj2*sj3;
      
    

    }
  
  return(sum);
  



}
