/* one_part.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <stdlib.h>
#include <math.h>

void one_particle(int i, int j, int method, int ii);
void Rone_particle(int i, int j, int method, int ii);

void red_one(int i, int j);
void Spin_Orbit(double mla, double msa, double mlb, double msb);
void Crystal_Field(double mla, double msa, double mlb, double msb);
void H_m(double mla, double msa, double mlb, double msb);
void precalc_onepart(void);



double lx(double mla, double mlb);
double ly(double mla, double mlb);
double sx(double msa, double msb);
double sy(double msa, double msb);
double Rt(int t);
double Sigmat(int t);
double RE_Atp(int t,int p), IM_Atp(int t,int p);
double Delta(double a, double b);

typedef double doublereal;
typedef struct { doublereal r, i; } doublecomplex;
typedef long int integer;

struct state {
double ml,ms;};

extern double RE0, IM0; 
extern struct state *Ang;
extern int electrons;
extern int **states, *states_tmp, signe, *un_paired;
extern doublecomplex *AP, *pre1, *prmx, *prmy, *prmz;
extern doublereal *rAP;
extern double Xi, R2, R4, R6, S2, S4, S6, Bx, By, Bz;
extern double  Bexx, Bexy, Bexz; /*pn*/
extern double Atp[20][20], Btp[20][20], lval;
extern int p_inc;
extern doublecomplex *mx,*my,*mz;
extern double Rmx, Imx, Rmy, Imy, Rmz, Imz;



extern double clebsg(double a, double b, double c, double xx, double yy, double zz, double gg, double hh, double pp, int mode);


void one_particle(int i, int j, int method, int ii)
{

  int k;
  
  RE0=IM0=0.0;
  Rmx=Imx=0.0;
  Rmy=Imy=0.0;
  Rmz=Imz=0.0;
  
  if(method==0)
    {
      for(k=0; k<electrons; k++)
	red_one( states[i][k], states_tmp[k]);

      AP[ii].r += RE0;
      AP[ii].i += IM0;
      mx[ii].r += Rmx;
      mx[ii].i += Imx;
      my[ii].r += Rmy;
      my[ii].i += Imy;
      mz[ii].r += Rmz;
      mz[ii].i += Imz;
/*               if(Rmz!=0.0 || RE0!=0.0) {
     printf("  %d  %12.8lf %12.8lf  \n",
          ii,mz[ii].r,AP[ii].r);} */

    }
  
  else if(method==1)
    {
      red_one(states[i][un_paired[1]], states_tmp[un_paired[1]]);
      
      AP[ii].r += RE0 * pow(-1.0,(double)signe);
      AP[ii].i += IM0 * pow(-1.0,(double)signe);

      mx[ii].r += Rmx * pow(-1.0,(double)signe);
      mx[ii].i += Imx * pow(-1.0,(double)signe);
      my[ii].r += Rmy * pow(-1.0,(double)signe);
      my[ii].i += Imy * pow(-1.0,(double)signe);
      mz[ii].r += Rmz * pow(-1.0,(double)signe);
      mz[ii].i += Imz * pow(-1.0,(double)signe);
 /*                if(Rmz!=0.0 || RE0!=0.0) {
     printf("  %d   %11.7lf %11.7lf\n",
          ii,mz[ii].r,AP[ii].r);} */

    }


}

void Rone_particle(int i, int j, int method, int ii)
{

  int k;
  
  RE0=0.0;
  
  if(method==0)
    {
      for(k=0; k<electrons; k++)
	red_one( states[i][k], states_tmp[k]);

      rAP[ii] += RE0;
     
    }
  
  else if(method==1)
    {
      red_one(states[i][un_paired[1]], states_tmp[un_paired[1]]);
      
      rAP[ii] += RE0 * pow(-1.0,(double)signe);
    }


}



void red_one(int i, int j)
{
  double mla, msa, mlb, msb;
  int n1;
  
  n1 = 2*(2*(int)lval+1);

  RE0 +=pre1[i*n1 + j].r;     
  IM0 +=pre1[i*n1 + j].i;     

  Rmx +=prmx[i*n1 + j].r;
  Imx +=prmx[i*n1 + j].i;
  Rmy +=prmy[i*n1 + j].r;
  Imy +=prmy[i*n1 + j].i;
  Rmz +=prmz[i*n1 + j].r;
  Imz +=prmz[i*n1 + j].i;

}


void Spin_Orbit(double mla, double msa, double mlb, double msb)
{
  double offdiag, diag;
  

  offdiag = lx(mla, mlb)*sx(msa, msb) - ly(mla, mlb)*sy(msa, msb);

  /* negative sign since i**2 = -1 */

  diag = mlb*msb*Delta(mla,mlb)*Delta(msa,msb);

  RE0 +=  Xi * (offdiag + diag);
  IM0 += 0.0;
}




double lx(double mla, double mlb)
{
  
  if(mla==mlb+1.0)
    return(0.5*sqrt(lval*(lval+1.0) - mlb*(mlb+1.0)));
  else if(mla==mlb-1.0)
    return(0.5*sqrt(lval*(lval+1.0) - mlb*(mlb-1.0)));
  else return(0.0);
  
}

double ly(double mla, double mlb)
{

  /* obs! this is the real ly without i */

  if(mla==mlb+1.0)
    return(-0.5*sqrt(lval*(lval+1.0) - mlb*(mlb+1.0)));
  else if(mla==mlb-1.0)
    return(0.5*sqrt(lval*(lval+1.0) - mlb*(mlb-1.0)));
  else return(0.0);

}

double sx(double msa, double msb)
{

  if(msa==msb+1.0)
    return(0.5*sqrt(0.75 - msb*(msb+1.0)));
  else if(msa==msb-1.0)
    return(0.5*sqrt(0.75 - msb*(msb-1.0)));
  else return(0.0);

}

double sy(double msa, double msb)
{

  if(msa==msb+1.0)
    return(-0.5*sqrt(0.75 - msb*(msb+1.0)));
  else if(msa==msb-1.0)
    return(0.5*sqrt(0.75 - msb*(msb-1.0)));
  else return(0.0);

}




void Crystal_Field(double mla, double msa, double mlb, double msb)
{

  int t,p;
  double trej1, trej2;
  double sumr=0.0, sumi=0.0;

  if(msa!=msb) 
    return;

  for (t=2; t<=2*(int)lval; t=t+2)
    {
      
      for (p=-t;p<=t; p++)
	{
	  trej1 = clebsg( lval,(double)t, lval, 0, 0, 0, 0, 0, 0, 1);
	  trej2 = clebsg( lval,(double)t, lval, -mla, (double)p, mlb, 0, 0, 0, 1);

	  sumr = sumr + RE_Atp(t,p) * Rt(t) * pow(-1.0, mla) * (2.0*lval+1.0) 
                      * trej1 * trej2;

	  sumi = sumi + IM_Atp(t,p) * Rt(t) * pow(-1.0, mla) * (2.0*lval+1.0)
	              * trej1 * trej2;

	}
    }


  RE0+=sumr;
  IM0+=sumi;

  return;

}

double RE_Atp(int t,int p)
{

  return(Atp[t][p+p_inc]*(1.0-Sigmat(t)));

}



double IM_Atp(int t,int p)
{

  return(Btp[t][p+p_inc]*(1.0-Sigmat(t)));

}

double Rt(int t)
{
  if(t==2) 
    return(R2);
  else if(t==4) 
    return(R4);
  else if(t==6) 
    return(R6);
  
  else 
    {printf("Error in Rt!\n"); exit(0);}

}

double Sigmat(int t)
{

  if(t==2) 
    return(S2);
  else if(t==4) 
    return(S4);
  else if(t==6) 
    return(S6);
  else 
    return(0.0);

}


void H_m(double mla, double msa, double mlb, double msb)
{
  
  double myb=0.5, gs=2*1.00115962, tesla=0.0000042551*219475.644;
  double sumr=0.0, sumi=0.0, sum=0.0;

  sumr=sumi=0.0;
  /*  Bz  */
  
  if(Bz!=0.0) {
    sumr = Delta(mla,mlb) * Delta(msa,msb) * myb * Bz * tesla 
      * ( mlb + gs * msb);

    Rmz += Delta(mla,mlb) * Delta(msa,msb) * ( mlb + gs * msb);
  }

  /*  Bx  */

  if(Bx!=0.0) {
    sum = Delta( msa, msb) 
      * (Delta( mla, mlb + 1) * sqrt( lval*(lval+1)-mlb*(mlb+1) ) +
	 Delta( mla, mlb - 1) * sqrt( lval*(lval+1)-mlb*(mlb-1) ) );

    Rmx += Delta( msa, msb)
      * (Delta( mla, mlb + 1) * sqrt( lval*(lval+1)-mlb*(mlb+1) ) +
         Delta( mla, mlb - 1) * sqrt( lval*(lval+1)-mlb*(mlb-1) ) ); 

    sum = sum +   gs * Delta( mla, mlb) 
      * (Delta( msb, -0.5) * Delta( msa,  0.5) +
	 Delta( msb,  0.5) * Delta( msa, -0.5) );

    Rmx = Rmx +  gs * Delta( mla, mlb)
      * (Delta( msb, -0.5) * Delta( msa,  0.5) +
         Delta( msb,  0.5) * Delta( msa, -0.5) );

    sum = sum * myb * Bx * tesla * 0.5;
    sumr = sumr + sum;
  }
 

  /* By */
 
  if(By!=0.0) {
    sumi = Delta( msa, msb) 
      * (Delta( mla, mlb - 1) * sqrt( lval*(lval+1)-mlb*(mlb-1) ) -
	 Delta( mla, mlb + 1) * sqrt( lval*(lval+1)-mlb*(mlb+1) ) );

    Imy += Delta( msa, msb)
      * (Delta( mla, mlb - 1) * sqrt( lval*(lval+1)-mlb*(mlb-1) ) -
         Delta( mla, mlb + 1) * sqrt( lval*(lval+1)-mlb*(mlb+1) ) );

    sumi = sumi + gs * Delta( mla, mlb) 
      * (Delta( msb,  0.5) * Delta( msa, -0.5) -
	 Delta( msb, -0.5) * Delta( msa,  0.5) );

     Imy = Imy + gs * Delta( mla, mlb)
      * (Delta( msb,  0.5) * Delta( msa, -0.5) -
         Delta( msb, -0.5) * Delta( msa,  0.5) );

    sumi = sumi * myb * By * tesla * 0.5;
  }
  
  RE0 += sumr;
  IM0 += sumi;


/* new part - exchange in molecular field approximation
   modification of H_m by P. Novak 2.3.2011 */

  sumr=sumi=0.0;
  /*  Bexz  */

  if(Bexz!=0.0)
    sumr = Delta(mla,mlb) * Delta(msa,msb) * myb * Bexz * tesla
      * gs * msb;

  /*  Bexx  */

  if(Bexx!=0.0) {
    sum = Delta( msa, msb);

    sum = sum +   gs * Delta( mla, mlb)
      * (Delta( msb, -0.5) * Delta( msa,  0.5) +
         Delta( msb,  0.5) * Delta( msa, -0.5) );

    sum = sum * myb * Bexx * tesla * 0.5;
    sumr = sumr + sum;
  }

  /* Bexy */

  if(Bexy!=0.0) {
    sumi = Delta( msa, msb);

    sumi = sumi + gs * Delta( mla, mlb)
      * (Delta( msb,  0.5) * Delta( msa, -0.5) -
         Delta( msb, -0.5) * Delta( msa,  0.5) );

    sumi = sumi * myb * Bexy * tesla * 0.5;
  }

  RE0 += sumr;
  IM0 += sumi;

/* P. Novak modification end */

  return;
}

double Delta(double a, double b)
{

  if(a==b) 
    return(1.0);
  else 
    return(0.0);
}



void precalc_onepart(void)
{
  int i,j,k,l,n1,n2,n3,len;
  double mla,mlb,mlc,mld,msa,msb,msc,msd;

  len=2*(2*(int)lval+1);
  n1=len; n2=n1*len; n3=n2*len;


  for(i=0;i<len;i++) {
    mla = Ang[i].ml;
    msa = Ang[i].ms;
    for(j=i;j<len;j++) {

      RE0=0.;IM0=0.;
      Rmx=0.;Imx=0.;
      Rmy=0.;Imy=0.;
      Rmz=0.;Imz=0.;
     
      mlb = Ang[j].ml;
      msb = Ang[j].ms;

      Spin_Orbit( mla, msa, mlb, msb);
      Crystal_Field( mla, msa, mlb, msb);
      H_m(mla, msa, mlb, msb); 
      
      pre1[i*n1 + j].r = RE0;
      pre1[i*n1 + j].i = IM0;
      
      pre1[j*n1 + i].r = RE0;
      pre1[j*n1 + i].i = -IM0;
      
      prmx[i*n1 + j].r = Rmx;
      prmx[i*n1 + j].i = Imx; 

      prmx[j*n1 + i].r = Rmx;
      prmx[j*n1 + i].i = -Imx;  

      prmy[i*n1 + j].r = Rmy;
      prmy[i*n1 + j].i = Imy; 

      prmy[j*n1 + i].r = Rmy;
      prmy[j*n1 + i].i = -Imy;  

      prmz[i*n1 + j].r = Rmz;
      prmz[i*n1 + j].i = Imz; 

      prmz[j*n1 + i].r = Rmz;
      prmz[j*n1 + i].i = -Imz;  

    }
  }
 
    
  



}

