/*=====================================================*/
/*                                                     */
/* refit: least square fit of exp. a calculation       */
/* based on Lanthanide2                                */
/* Atomic Program by Sverker Edvardsson & Daniel Aberg */
/*                                                     */
/*=====================================================*/
/*                                                     */
/* Only equivalent electrons allowed !                 */
/*                       N                             */
/* Program general for nl  configuration except        */
/* for the CI parts of Trees parameters i.e.           */
/* functions t2T2-t8T8. Those are valid for            */
/* f-electrons only.                                   */
/*                                                     */
/*=====================================================*/
/*                                                     */
/* atom/material dependencies are in files:            */
/*                                                     */
/*   fort.7=refit.inp                                         */
/*   fort.8=cfp.inp                                           */
/*   vk1k2k3                                        */
/*   Altp.inp                                          */
/*   Ak1k2ltp.inp                                      */
/*                                                     */
/*=====================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <stdlib.h>
#include <math.h>
#include <zlib.h>

int STORE;
int RANGE;
int STATES, electrons;
double lval;
int no_el;

typedef double doublereal;
typedef struct { double r, i; } doublecomplex;
typedef long int integer;

struct state {
  double ml, ms;} *Ang;
double RE0, IM0, RE1, IM1, Rmx, Imx, Rmy, Imy, Rmz, Imz;
double *pre2;
doublecomplex *pre1, *prmx, *prmy, *prmz;

int min_p = -1;
int *st_cq, *in_cq, cq_start, *in_en;
double  *st_mj;
int **states, *states_tmp, *un_paired, signe;
double *energies, *RE_EIGC;
doublecomplex *EIGC;
/* Psi(i) = sum_j eigc[i*STATES+j] * {j} */
doublecomplex *AP;
doublecomplex *AP1;
doublecomplex *AP2;
doublecomplex *AP3;
doublereal *rAP;


doublecomplex *mx,*my,*mz;       /* magnetic moments of basis states */


int method_judd(int i, int j);
void Malloc(void);
void readeigencoeff(void);


  FILE *pout;
  FILE *hap1;
  FILE *hap2;
  FILE *hap3;
  FILE *mapx;
  FILE *mapy;
  FILE *mapz;

extern void one_particle(int i, int j, int method, int ii);
extern void two_particle(int i, int j, int method, int ii);
extern void Rone_particle(int i, int j, int method, int ii);
extern void Rtwo_particle(int i, int j, int method, int ii);
extern void three_particle(int i, int j, int ii);
extern void Rthree_particle(int i, int j, int ii);
extern void precalc_twopart(void);
extern void precalc_onepart(void);
extern double clebsg(double a, double b, double c, 
		     double xx, double yy, double zz, 
		     double gg, double hh, double pp, int mode);

void readatp(void);
double Atp[20][20], Btp[20][20];
/* double RE_Altp[8][20][20], IM_Altp[8][20][20];*/
double RE_Ak1k2ltp[7][7][15][20][20], IM_Ak1k2ltp[7][7][15][20][20];
int p_inc, q3_inc;
 
extern void lu(void);


extern void calc_Vkkpkb(int method, int i, int j);

extern double t2T2(void), t3T3(void), t4T4(void);
extern double t6T6(void), t7T7(void), t8T8(void);
extern void load_Vkkpkb(void), save_Vkkpkb(int i, int j);

void load_pauli(void);
void sort_i(int numbers[], int order[], int no);
void sort_d(double numbers[], int order[], int no);


void construct(void);
void loop(int slot);
double fac(int no);
int PAULI[40];

FILE *pek_save;

void readinp(void);
double R2, R4, R6, S2, S4, S6;
double F2, F4, F6, Xi;
double Alpha, Beta, Gamma;
double T2,T3,T4,T6,T7,T8;
double Mk[10], Pk[10];
double Bx, By, Bz;
double Bexx, Bexy, Bexz;  /*pn*/
int windowtype, ground_no, polar, load, Dop;
double hi_en, lo_en, hi_wav, lo_wav, T;
double fwhm;
extern struct
{
   int ii, jj, kk, ll;
} ijk_;


int refitc_(int argc, char **argv)
{
  int a,b,i,j,method, ii;
  long int elements=0;
  char name[40];
  int dummy, I, J;
  FILE *pek;
  double temp;

  load=ijk_.ii;     

  readinp();  
  readatp();
  load_pauli();

  ijk_.jj=STATES;
  ijk_.kk=RANGE;
  fprintf(pout,"\n\nload is %d #STATES=%d #RANGE=%d #electrons=%d\n\n",load, STATES, RANGE, electrons);

  if((load==1)&&(STORE!=1))
    {fprintf(pout,"\nOverriding refit.inp; STORE=1\n");STORE=1;}

  if((load==0)&&(0.5*STATES*(STATES+1.0)>50000.0))
    {
      if(lval==3.0) {
	fprintf(pout,"\n\nCalculating %ld three-particle elements...\n",(long)(0.5*STATES*(STATES+1.0)));
      }
    }

  (hap1=fopen("fort.41","w"));
  (hap2=fopen("fort.42","w"));
  (hap3=fopen("fort.43","w"));
  (mapx =fopen("fort.44","w"));
  (mapy =fopen("fort.45","w"));
  (mapz =fopen("fort.46","w"));

  if(((load==1))&&(lval==3.0))
    {
      if(min_p==-1)
	sprintf(name,"vk1k2k3_%d",electrons);
      else
	sprintf(name,"vk1k2k3_%d_%d",electrons,min_p);
      if((pek_save=fopen(name,"r"))==NULL) 
      {printf("Error in %s!\n",name); exit(0);}
      
      load_Vkkpkb();
    }
  else if(load==0)
    {
      if(lval==3.0) {
	if(min_p==-1)
	  sprintf(name,"vk1k2k3_%d",electrons);
	else
	  sprintf(name,"vk1k2k3_%d_%d",electrons,min_p);
      if((pek_save=fopen(name,"w"))==NULL) 
	{printf("Error in %s!\n",name); exit(0);}
      }
      else
	{fprintf(pout,"3-particle ops. is implemented for f-electrons only\n\n");
	}
    }

  if(load==1) {
    fprintf(pout,"pre-calculating one- and two-particle elements\n\n");  
    
    precalc_onepart();
    
    if(ijk_.ll > 1){
    precalc_twopart();}
  
  }
  
  ii=0;
  if((load<2)&&(min_p==-1))
    {
      for(j=0; j<STATES; j++)
	{
	  for(i=0; i<=j; i++)       
	    {
	      if(load==0)
		{		  
		  ii++;
		  method=method_judd(i,j);
		  calc_Vkkpkb(method, i, j); 
		  save_Vkkpkb( i, j);     
		}
	      else 
		{  
		  method = method_judd(i,j);
		  
		  AP[ii].r  = AP[ii].i = 0.0;
                  mx[ii].r  = mx[ii].i = 0.0;
                  my[ii].r  = my[ii].i = 0.0;
                  mz[ii].r  = mz[ii].i = 0.0;
		  one_particle(i, j, method, ii);
                  AP1[ii].r = AP[ii].r;
                  AP1[ii].i = AP[ii].i;
                  if(AP1[ii].r!=0.0 || AP1[ii].i!=0.0) {
		    fprintf(hap1,"%d %d %5.12e %5.12e\n",i+1,j+1,AP1[ii].r,AP1[ii].i);}
		  if(mx[ii].r!=0.0 || mx[ii].i!=0.0) {
		    fprintf(mapx," %d %d %5.12e %5.12e\n",i+1,j+1,mx[ii].r,mx[ii].i);
		  }
		  if(my[ii].r!=0.0 || my[ii].i!=0.0) {
		    fprintf(mapy," %d %d %5.12e %5.12e\n",i+1,j+1,my[ii].r,my[ii].i);
		  }
		  if(mz[ii].r!=0.0 || mz[ii].i!=0.0) {
		    fprintf(mapz," %d %d %5.12e %5.12e\n",i+1,j+1,mz[ii].r,mz[ii].i);
		  }
                  if(ijk_.ll > 1){
		   two_particle(i, j, method, ii);
                   AP2[ii].r = AP[ii].r-AP1[ii].r;
                   AP2[ii].i = AP[ii].i-AP1[ii].i;
                   if(AP2[ii].r!=0.0 || AP2[ii].i!=0.0) {
		    fprintf(hap2,"%d %d %5.12e %5.12e\n",i+1,j+1,AP2[ii].r,AP2[ii].i);}
		   three_particle(i, j, ii); 
                   AP3[ii].r = AP[ii].r-AP1[ii].r-AP2[ii].r;
                   AP3[ii].i = AP[ii].i-AP1[ii].i-AP2[ii].i;
                   if(AP3[ii].r!=0.0 || AP3[ii].i!=0.0) {
		    fprintf(hap3,"%d %d %5.12e %5.12e\n",i+1,j+1,AP3[ii].r,AP3[ii].i);}
		  }
		  
		  
		  ii++;
		}
	    }
	}
       fclose(hap1);  
       fclose(hap2);  
       fclose(hap3);  

      fprintf(pout,"done\n");
    }


  if( load==0 || load==1 )
    if(lval==3.0)
      fclose(pek_save);

       fclose(mapx);
       fclose(mapy);
       fclose(mapz);
  
  if( load==1 )
    {
      free(AP);free(pre2);free(pre1);free(prmx);free(prmy);free(prmz);
    }

  
  free(st_mj);
  free(st_cq);
  free(in_cq);
  free(in_en);
  free(states);
  free(states_tmp);
  free(Ang);
  return(0);
}



/* ordering of Determinantal Product States */

int method_judd(int i, int j)
{

  int k, l, m, temp;
 
  m=0;
  for (k=0; k<electrons; k++)
    states_tmp[k] = states[j][k];

  signe=0;

  for (k=0; k<electrons; k++)
    {
      for (l=0;l<electrons; l++)
	{
	  if(states[i][k]==states_tmp[l])
	    {
	      if(l!=k)
		{ 
		  temp = states_tmp[l];
		  states_tmp[l]=states_tmp[k];
		  states_tmp[k]=temp;
		  signe++;
		}
	      break;
	    }
	} 
    }

  for(k=0; k<electrons; k++)
    if(states[i][k]!=states_tmp[k])
      { m++;   un_paired[m]=k;}


  return(m);

}





void readatp(void)
{

  FILE *pekatp;
  int dummy1,dummy2;
  int t,p,q,k,k1,k2;
  double real,imag,dummy;
  fprintf(pout, " \n Crystal field coefficients \n");
  fprintf(pout, "    t    p     Btp real      Btp imag \n");

  if((pekatp=fopen("fort.8","r"))==NULL)  /* fort.8 = orig. cfp.inp */ 
    {puts("Error in fort.8 = orig. cfp.inp!"); exit(0);}

  p_inc = 7;
  q3_inc = 12;


  for(t=0;t<=7; t++)
    {
      for(p=-t;p<=t;p++)
	{
	  if(fscanf( pekatp, "%d%d%lf%lf",&dummy1,&dummy2,&real,&imag))
	    {
             dummy = real*real + imag*imag;
             if(dummy!=0.) {
             fprintf( pout, " %4d %4d %12.3lf %12.3lf \n",dummy1,dummy2,real,imag);
	     }      
	      /* pseudo cm-1, will be upon mult. with <r**t> */

	      Atp[t][p+p_inc]=real;/*219475.644; */
	      Btp[t][p+p_inc]=imag;/*219475.644;*/
	    }
	  else {puts("End of file."); exit(0);}
	}
    }
  
  if(min_p!=-1) {
    min_p=0;
    
    if((R2!=0.0)||(R4!=0.0)||(R6!=0.0)) {
      for(t=2;t<7;t=t+2)
	{
	  
	  if(Btp[t][p_inc] != 0.0) 
	    {printf("\ncfp.inp not hermitean!\nCheck t=%d p=%d\n\n",t,0);
	    exit(0);}
	  
	  for(p=1;p<=t;p++)
	    {
	      if((Atp[t][p+p_inc]!=0.0)||(Btp[t][p+p_inc]!=0.0))
		{
		  if(min_p==0) min_p = p;
		  if(p<min_p) min_p = p;
		}
	      
	      if((Atp[t][p+p_inc] != pow(-1.0,(double)p)*Atp[t][-p+p_inc] )||
		 (Btp[t][p+p_inc] != -pow(-1.0,(double)p)*Btp[t][-p+p_inc])) 
		{printf("\ncfp.inp not hermitean!\nCheck t=%d p=%d\n\n",t,p);
		exit(0);}
	    }
	}
    }
    else {
      for(t=0;t<8;t++)
	for(p=-t;p<=t;p++)
	  Atp[t][p+p_inc]=Btp[t][p+p_inc]=0.0;
      min_p=0;
    }
  

    if((Bx!=0.0)||(By!=0.0)||(Bz!=0.0)) min_p=1;
  }


  if(R2==-1.0&&R4==-1.0&&R6==-1.0)
    {
      
      printf("Hcf is expanded in unit tensors.\n");
      
      
      dummy = pow(-1.0,lval)*(2.0*lval+1.0)*
	clebsg( lval, 2.0, lval, 0, 0, 0, 0, 0, 0, 1);
      R2 = 1.0 / dummy;
      dummy = pow(-1.0,lval)*(2.0*lval+1.0)*
	clebsg( lval, 4.0, lval, 0, 0, 0, 0, 0, 0, 1);
      R4 = 1.0 / dummy;
      dummy = pow(-1.0,lval)*(2.0*lval+1.0)*
	clebsg( lval, 6.0, lval, 0, 0, 0, 0, 0, 0, 1);
      R6 = 1.0 / dummy;
      
    }


  
  fclose(pekatp);
  
  if((dummy1!=7)||(dummy2!=7))
    {
      printf("End of file or error in cfp.inp!!!\n");
      exit(0);
    }

  if(load<1)
    return;

/*  if((pekatp=fopen("Altp.inp","r"))==NULL) 
    {puts("Error in Altp.inp!"); exit(0);}

  p_inc = 7;
  q=0;
  

  for(k=0;k<=6;k++)
    for(t=abs(k-1);t<=k+1;t++)
      for(p=-t;p<=t;p++) 
	{
	  fscanf( pekatp, "%d%d%d%lf%lf",&dummy1,&dummy2,&dummy2,&real,&imag);
	    
	  RE_Altp[k][t][p+p_inc]=real;
	  IM_Altp[k][t][p+p_inc]=imag;
	  
	}
  
  fclose(pekatp); */

  if(Dop!=2) return;
  
  if((pekatp=fopen("Ak1k2ltp.inp","r"))==NULL) 
    {puts("Error in Ak1k2ltp.inp!"); exit(0);}

  
  while(fscanf( pekatp, "%d%d%d%d%d%lf%lf",&k1,&k2,&k,&t,&p,&real,&imag)!=EOF) 
    {
      RE_Ak1k2ltp[k1][k2][k][t][p+p_inc]=real;
      IM_Ak1k2ltp[k1][k2][k][t][p+p_inc]=imag;
    }
  

  fclose(pekatp);
  
}



void load_pauli(void)
{

  int i,j,dummy;
  double mll, mss, sum;
  int **tmp;
  double *tmp_mj;

  i=0;

  if(!(Ang = malloc( 2*(2*(int)lval+1)  *sizeof(struct state)))) 
    {printf("No mem Ang!\n"); exit(0);}
  
  for(mll=-lval;mll<=lval;mll+=1.0)
    for(mss=-0.5;mss<=0.5;mss+=1.0)
      { 
	Ang[i].ml = mll;
	Ang[i].ms = mss;
	i++;
      }

  STATES = (int)( fac(i)/(fac(i-electrons)*fac(electrons)));

  Malloc();
  no_el = 2*(2*(int)lval+1);
  construct();


  if(min_p==-1) return;
 

  for(i=0;i<STATES;i++) {
    sum = 0.0;
    for(j=0;j<electrons;j++) 
      sum += Ang[states[i][j]].ml + Ang[states[i][j]].ms;
    
    st_mj[i] = sum;

      
    if(min_p>0) {
      
      if(electrons%2==0)
	{
	  st_cq[i] = ((int)sum)%min_p;
	
	if(abs(st_cq[i]+min_p)<=abs(st_cq[i]))
	  st_cq[i]= st_cq[i]+min_p;
	if(abs(st_cq[i]-min_p)<=abs(st_cq[i]))
	  st_cq[i]= st_cq[i]-min_p;
	
	if(st_cq[i]+min_p==abs(st_cq[i])) st_cq[i]=abs(st_cq[i]);
	
	}
      else {
	st_cq[i] = ((int)(2.0*sum))%(2*min_p);
	
	if(abs(st_cq[i]+2*min_p)<=abs(st_cq[i]))
	  st_cq[i]= st_cq[i]+2*min_p;
	if(abs(st_cq[i]-2*min_p)<=abs(st_cq[i]))
	  st_cq[i]= st_cq[i]-2*min_p;
	if(st_cq[i]+2*min_p==abs(st_cq[i])) st_cq[i]=abs(st_cq[i]);
      }
      
      // printf("%d\n",st_cq[i]);getchar();
      
    }
    else if(min_p==0){
      if(electrons%2==0)
	st_cq[i]=abs((int)sum);
      else {
	if(sum==0.5) st_cq[i]=1;
	else st_cq[i]=100;
      }
    }
    
      
  }
  
  for(i=0;i<STATES;i++) in_cq[i]=i;
  if(min_p>=0)
    sort_i(st_cq, in_cq,STATES);
  
  if(!(tmp_mj = malloc(STATES*sizeof(double)))) 
    {printf("No mem tmp!\n"); exit(0);}

  for(i=0;i<STATES;i++)
    tmp_mj[i]=st_mj[in_cq[i]];
   for(i=0;i<STATES;i++)
     st_mj[i]=tmp_mj[i];
  
  while(cq_start<STATES)
    {
      dummy=st_cq[cq_start];
      
      /* counts how big submatrix is */
      i=0; while(dummy-st_cq[i+cq_start]==0) i++;
      sort_d(&st_mj[cq_start],&in_cq[cq_start],i);
      cq_start+=i;
    }
  
  if(!(tmp = malloc(STATES*sizeof(int)))) 
    {printf("No mem tmp!\n"); exit(0);}
  for(i=0;i < STATES; i++)
    if(!( tmp[i]=malloc(electrons * sizeof(int)))) 
      {printf("No mem tmp!\n"); exit(0);}
    
  for(i=0;i<STATES;i++)
    {
      // tmp_mj[i]=st_mj[in_cq[i]];
      for(j=0;j<electrons;j++)
	tmp[i][j] = states[in_cq[i]][j];
    }
  for(i=0;i<STATES;i++)
    {
      //st_mj[i]=tmp_mj[i];
      for(j=0;j<electrons;j++)
	states[i][j] = tmp[i][j];
    }


  



  free(tmp);
  free(tmp_mj);
  

}


double fac(int no)
{
  int dummy;
  double facu=1.0;

  for(dummy=1;dummy<=no;dummy++)
    facu=facu*dummy;

  return(facu);
}

void construct(void)
{

  STATES = 0;
  loop(1);

}

void loop(int slot)
{
  int i,begin,end;
  
  
  begin = PAULI[slot-1];
     
  if(slot==1)
    {begin = 0;}

  end = no_el - (electrons - slot);
  PAULI[0]=0;
  
  for( PAULI[slot] = begin+1; 
       PAULI[slot] <= end; 
       PAULI[slot]++)
    {
      if(slot==electrons)
	{
	  /* now we have an accepted state, so flush it */
	  for(i=1;i<=electrons;i++)
	    states[STATES][i-1] = PAULI[i]-1;

	  STATES++;
	}
      if(slot<electrons)
	{      
	  slot++;
	  loop(slot);
	  PAULI[slot]--;
	  slot--;
	}
      
    }
   


}

void readinp(void)
{
  FILE *pek;
  
/* fort.7=orig. lanthanide.inp */
  if((pek=fopen("fort.7","r"))==NULL) 
    {puts("Error in fort.7=refit.inp=orig. lanthanide.inp!"); exit(0);}

  fscanf( pek,"%d %lf %d %d",&electrons,&lval,&RANGE,&STORE);
  fscanf(pek,"%lf%lf%lf%lf%lf%lf", &R2, &R4, &R6, &S2, &S4, &S6);
  fscanf(pek,"%lf%lf%lf%lf", &F2, &F4, &F6, &Xi);
  fscanf(pek,"%lf%lf%lf", &Alpha, &Beta, &Gamma);
  fscanf(pek,"%lf%lf%lf%lf%lf%lf", &T2, &T3, &T4, &T6, &T7, &T8);
  fscanf(pek,"%lf%lf%lf%lf%lf%lf", &Mk[0], &Mk[2], &Mk[4], &Pk[2], &Pk[4], &Pk[6]);
  
  fscanf(pek,"%lf%lf%lf", &Bx, &By, &Bz);
  fscanf(pek,"%lf%lf%lf", &Bexx, &Bexy, &Bexz);  /*pn*/
  
/*  fscanf(pek,"%d %d %lf", &Dop, &polar, &T);
  fscanf( pek,"%d %lf %lf",&windowtype, &lo_wav,&hi_wav);
  fscanf( pek,"%lf %lf %lf",&lo_en,&hi_en,&fwhm);*/
 
  fclose(pek);

  (pout=fopen("outputc","w"));
  fprintf(pout, "   Nel     L     RANGE    STORE \n");
  fprintf(pout, "  %3d %6.1lf %7d %7d \n\n",electrons, lval, RANGE, STORE);
  fprintf(pout, "    R2       R4      R6      S2      S4       S6 \n");
  fprintf(pout,"%7.2lf %7.2lf %7.2lf %7.2lf %7.2lf %7.2lf  \n\n", R2, R4, R6, S2, S4, S6);
  fprintf(pout, "      F2        F4        F6          Xi  \n");
  fprintf(pout,"%10.1lf %10.1lf %10.1lf %10.1lf  \n\n", F2, F4, F6, Xi);
  fprintf(pout, "     Alpha       Beta      Gamma \n");
  fprintf(pout,"%10.1lf %10.1lf %10.1lf  \n\n", Alpha, Beta, Gamma);
  fprintf(pout, "      T2        T3        T4        T6        T7       T8 \n");
  fprintf(pout,"%9.2lf %9.2lf %9.2lf %9.2lf %9.2lf %9.2lf  \n\n", T2, T3, T4, T6, T7, T8);
  fprintf(pout,"     Mk[0]     Mk[2]     Mk[4]     Pk[2]     Pk[4]     Pk[6]  \n");
  fprintf(pout,"%9.2lf %9.2lf %9.2lf %9.2lf %9.2lf %9.2lf  \n\n", Mk[0], Mk[2], Mk[4], Pk[2], Pk[4], Pk[6]);
  fprintf(pout,"       Bx         By         Bz  (external magnetic field T )\n");
  fprintf(pout,"  %9.2lf %9.2lf %9.2lf  \n\n", Bx, By, Bz);
  fprintf(pout,"      Bex_x      Bex_y      Bex_z (exchange field T) \n");  /*pn*/
  fprintf(pout,"  %9.2lf %9.2lf %9.2lf  \n\n", Bexx, Bexy, Bexz);  /*pn*/

  fprintf(pout,"   Dop   polar    T \n");
  fprintf(pout,"%6d %6d %7.2lf \n\n", Dop, polar, T);
  fprintf(pout," windowtype lo_wav  hi_wav \n");
  fprintf(pout,"%8d %7.2lf %7.2lf \n\n",windowtype, lo_wav,hi_wav);
  fprintf(pout,"    lo_en     hi_en     fwhm \n");
  fprintf(pout,"%9.2lf %9.2lf %9.2lf \n",lo_en,hi_en,fwhm);

}

void Malloc(void)
{
  int i, size;
  double dummy;

  if(RANGE==0) RANGE=STATES;

  /* states[STATES][electrons] */

  if(!(states = malloc(STATES*sizeof(int*)))) 
    {printf("No mem states!\n"); exit(0);}
  for(i=0;i < STATES; i++)
    if(!( states[i]=malloc(electrons * sizeof(int)))) 
      {printf("No mem states!\n"); exit(0);}
  
  if(!(states_tmp = malloc(electrons*sizeof(int)))) 
    {printf("No mem states_tmp!\n"); exit(0);}
  
  if(!(un_paired = malloc(electrons * sizeof(int)))) 
    {printf("No mem_un_paired!"); exit(0);}

  if(!(energies = malloc(STATES*sizeof(double))))
    {printf("No mem_energies!"); exit(0);}

  if(load==1 )
    {
      dummy = (((double)STATES*((double)STATES+1.0))/2.0);
      size = (integer)dummy;


      if(!(mx = malloc( 2*size*sizeof(doublecomplex))))
        {printf("No mem_mx!");  exit(0);}
      if(!(my = malloc( 2*size*sizeof(doublecomplex))))
        {printf("No mem_my!");  exit(0);}
      if(!(mz = malloc( 2*size*sizeof(doublecomplex))))
        {printf("No mem_mz!");  exit(0);}

      if(!(AP = malloc( size*sizeof(doublecomplex))))    
	{printf("No mem_ap!");  exit(0);}
      if(!(AP1 = malloc( size*sizeof(doublecomplex))))    
	{printf("No mem_ap!");  exit(0);}
      if(!(AP2 = malloc( size*sizeof(doublecomplex))))    
	{printf("No mem_ap!");  exit(0);}

      if(!(AP3 = malloc( size*sizeof(doublecomplex))))    
	{printf("No mem_ap!");  exit(0);}

      
      size = (int)pow(2*(2*(int)lval+1),4.0);
      

      if(!(pre2 = malloc( size*sizeof(double))))    
	{printf("No mem pre2!");  exit(0);}
      
      size = (int)pow(2*(2*(int)lval+1),2.0);
      if(!(pre1 = malloc( 2*size*sizeof(doublecomplex))))    
	{printf("No mem pre1!");  exit(0);}
      if(!(prmx = malloc(2*size*sizeof(doublecomplex))))
        {printf("No mem_prmx!");  exit(0);}
      if(!(prmy = malloc(2*size*sizeof(doublecomplex))))
        {printf("No mem_prmy!");  exit(0);}
      if(!(prmz = malloc(2*size*sizeof(doublecomplex))))
        {printf("No mem_prmz!");  exit(0);}
      
    }



  if(load==1 )
    {
      if(!(EIGC = malloc(STATES*STATES*sizeof(doublecomplex)))) 
	{printf("No mem EIGC!\n"); exit(0);}

    }

  
  if(!(st_mj= malloc(STATES*sizeof(double))))
    {printf("No st_mj!"); exit(0);}


  if(!(st_cq= malloc(STATES*sizeof(int))))
    {printf("No st_cq!"); exit(0);}

  if(!(in_cq= malloc(STATES*sizeof(int))))
    {printf("No in_cq!"); exit(0);}

  if(!(in_en= malloc(STATES*sizeof(int))))
    {printf("No in_cq!"); exit(0);}

}


void readeigencoeff(void)
{
  int i,j;
  FILE *pek2;
  int gn,gi,gj,gk;
  char buf[1], line[60], flyt[30];
  double dummy1,dummy2;

  FILE *pek;

  if((pek=fopen("eigenfunctions","r"))==NULL) 
	{puts("Error in eigenfunctions!"); exit(0);}
  
  

  fclose(pek);
  /*--------------------------------------------*/




  if((pek2=fopen("energies","r"))==NULL) 
    {puts("Error in energies!"); exit(0);}
  
  for(i=0; i<RANGE;i++)
    fscanf(pek2,"%lf",&energies[i]);
    
  fclose(pek2);


}



void sort_i(int numbers[], int order[], int no)
{
  int i, j;
  int temp;
  int itemp;

 
  for (i = (no - 1); i >= 0; i--)
  {
    for (j = 1; j <= i; j++)
    {
      if (numbers[j-1] > numbers[j])
      {
        temp = numbers[j-1];
        itemp = order[j-1];

        numbers[j-1] = numbers[j];
        order[j-1] = order[j];

        numbers[j] = temp;
        order[j] = itemp;
      }
    }
  }


}

void sort_d(double numbers[], int order[], int no)
{
  int i, j;
  double temp;
  int itemp;

  //  for(i=0;i<no;i++)
  // in_cq[i]=i;
 
  for (i = (no - 1); i >= 0; i--)
  {
    for (j = 1; j <= i; j++)
    {
      if (numbers[j-1] > numbers[j])
      {
        temp = numbers[j-1];
        itemp = order[j-1];

        numbers[j-1] = numbers[j];
        order[j-1] = order[j];

        numbers[j] = temp;
        order[j] = itemp;
      }
    }
  }


}

