/* three_part.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <stdlib.h>
#include <math.h>
#include <zlib.h>

void three_part(int method, int i, int j);

void determinant(double ml1, double ms1, 
		 double ml2, double ms2, 
		 double ml3, double ms3, 
		 double ml4, double ms4, 
		 double ml5, double ms5, 
		 double ml6, double ms6);

void red_determinant(double ml1, double ms1, 
		     double ml1p, double ms1p, 
		     double ml2, double ms2, 
		     double ml2p, double ms2p, 
		     double ml3, double ms3, 
		     double ml3p, double ms3p,
		     double factor);

double triple_scalar(double k1, double k2, double k3,
		     double ml1, double ms1, 
		     double ml1p, double ms1p, 
		     double ml2, double ms2, 
		     double ml2p, double ms2p, 
		     double ml3, double ms3, 
		     double ml3p, double ms3p);

void calc_Vkkpkb(int method, int i, int j);

void three_particle(int i, int j, int ii);
void Rthree_particle(int i, int j, int ii);
double t2T2(void), t3T3(void), t4T4(void);
double t6T6(void), t7T7(void), t8T8(void);
void save_Vkkpkb(int i, int j);
void load_Vkkpkb(void);



double POW(int a);

typedef double doublereal;
typedef struct { doublereal r, i; } doublecomplex;
typedef long int integer;

struct state {
double ml, ms;
};

extern struct state *Ang;
extern int electrons;
extern int **states, *states_tmp, signe, *un_paired;
extern FILE *pek_save;
extern double T2,T3,T4,T6,T7,T8,lval;
extern doublecomplex *AP;
extern doublereal *rAP;
extern int REAL;


double Vkkpkb222, Vkkpkb224, Vkkpkb244, Vkkpkb246, Vkkpkb444;
double Vkkpkb446, Vkkpkb266, Vkkpkb466, Vkkpkb666;
int vi, vj;

extern double clebsg(double a, double b, double c, double xx, double yy, double zz, double gg, double hh, double pp, int mode);

void three_particle(int i, int j, int ii)
{

  if(lval==3)
    if((vi==i)&&(vj==j))
      {
	AP[ii].r += t2T2() + t3T3() + t4T4() + t6T6() + t7T7() + t8T8();
	load_Vkkpkb();
      }

}

void Rthree_particle(int i, int j, int ii)
{

  if(lval==3)
    if((vi==i)&&(vj==j))
      {
	rAP[ii] += t2T2() + t3T3() + t4T4() + t6T6() + t7T7() + t8T8();
	load_Vkkpkb();
      }

}



double t2T2(void)
{

  if(electrons<3) return(0.0);

 

  return(T2*
	 (sqrt(605.0/5292.0)        * Vkkpkb222 -
	  sqrt(6760.0/43659.0)      * Vkkpkb224 -
	  sqrt(1805.0/391314.0)     * Vkkpkb244 -
	  sqrt(4160.0/754677.0)     * Vkkpkb246 +
	  sqrt(55016.0/717409.0)    * Vkkpkb444 -
	  sqrt(195.0/204974.0)      * Vkkpkb446 +
	  sqrt(1625.0/143748.0)     * Vkkpkb266 +
	  sqrt(88400.0/1185921.0)   * Vkkpkb466 +
	  sqrt(29393.0/790614.0)    * Vkkpkb666));

}

double t3T3(void)
{

  if(electrons<3) return(0.0);

  return(T3*
	 (sqrt(32761.0/889056.0)    * Vkkpkb222 +
	  sqrt(33.0/1372.0)         * Vkkpkb224 -
	  sqrt(4.0/33957.0)         * Vkkpkb244 -
	  sqrt(13.0/264.0)          * Vkkpkb246 +
	  sqrt(49972.0/622545.0)    * Vkkpkb444 +
	  sqrt(52.0/1089.0)         * Vkkpkb446 +
	  sqrt(325.0/199584.0)      * Vkkpkb266 -
	  sqrt(442.0/12705.0)       * Vkkpkb466 +
	  sqrt(205751.0/784080.0)   * Vkkpkb666));
}

double t4T4(void)
{

  if(electrons<3) return(0.0);

  return(T4*
	 (sqrt(3575.0/889056.0)     * Vkkpkb222 -
	  sqrt(325.0/37044.0)       * Vkkpkb224 -
	  sqrt(54925.0/373527.0)    * Vkkpkb244 +
	  sqrt(625.0/26136.0)       * Vkkpkb246 +
	  sqrt(92480.0/1369599.0)   * Vkkpkb444 +
	  sqrt(529.0/11979.0)       * Vkkpkb446 +
	  sqrt(6889.0/2195424.0)    * Vkkpkb266 -
	  sqrt(10880.0/251559.0)    * Vkkpkb466 -
	  sqrt(79135.0/1724976.0)   * Vkkpkb666));

}

double t6T6(void)
{

if(electrons<3) return(0.0);

 return(T6*
	(-sqrt(1573.0/8232.0)      * Vkkpkb222 -
	 sqrt(15028.0/305613.0)    * Vkkpkb224 +
	 sqrt(4693.0/12326391.0)   * Vkkpkb244 +
	 sqrt(1568.0/107811.0)     * Vkkpkb246 -
	 sqrt(297680.0/5021863.0)  * Vkkpkb444 -
	 sqrt(49.0/395307.0)       * Vkkpkb446 -
	 sqrt(1.0/223608.0)        * Vkkpkb266 -
	 sqrt(174080.0/8301447.0)  * Vkkpkb466 +
	 sqrt(79135.0/175692.0)    * Vkkpkb666));

}

double t7T7(void)
{

if(electrons<3) return(0.0);

 return(T7*
	(sqrt(264407.0/823200.0)   * Vkkpkb222 +
	 sqrt(28717.0/2778300.0)   * Vkkpkb224 -
	 sqrt(1273597.0/28014525.0)* Vkkpkb244 +
	 sqrt(841.0/1960200.0)     * Vkkpkb246 -
	 sqrt(719104.0/2282665.0)  * Vkkpkb444 -
	 sqrt(1369.0/35937.0)      * Vkkpkb446 +
	 sqrt(625.0/81312.0)       * Vkkpkb266 -
	 sqrt(8704.0/3773385.0)    * Vkkpkb466 +
	 sqrt(15827.0/319440.0)    * Vkkpkb666));
 
}

double t8T8(void)
{

  if(electrons<3) return(0.0);

  return(T8*
	 (sqrt(21879.0/274400.0)    * Vkkpkb222 -
	  sqrt(37349.0/926100.0)    * Vkkpkb224 +
	  sqrt(849524.0/9338175.0)  * Vkkpkb244 -
	  sqrt(17.0/653400.0)       * Vkkpkb246 -
	  sqrt(73644.0/2282665.0)   * Vkkpkb444 +
	  sqrt(68.0/11979.0)        * Vkkpkb446 +
	  sqrt(1377.0/27104.0)      * Vkkpkb266 -
	  sqrt(103058.0/1257795.0)  * Vkkpkb466 -
	  sqrt(8379.0/106480.0)     * Vkkpkb666));

}

void save_Vkkpkb(int i, int j)
{


  if((Vkkpkb222==0.0)&&(Vkkpkb224==0.0)&&
     (Vkkpkb244==0.0)&&(Vkkpkb246==0.0)&&
     (Vkkpkb444==0.0)&&(Vkkpkb446==0.0)&&
     (Vkkpkb266==0.0)&&(Vkkpkb466==0.0)&&
     (Vkkpkb666==0.0))
    return;
		      
  fprintf(pek_save,"%d %d ",i,j);
  
  fprintf(pek_save,"%10.12le ",Vkkpkb222);
  fprintf(pek_save,"%10.12le ",Vkkpkb224);
  fprintf(pek_save,"%10.12le ",Vkkpkb244);
  fprintf(pek_save,"%10.12le ",Vkkpkb246);
  fprintf(pek_save,"%10.12le ",Vkkpkb444);
  fprintf(pek_save,"%10.12le ",Vkkpkb446);
  fprintf(pek_save,"%10.12le ",Vkkpkb266);
  fprintf(pek_save,"%10.12le ",Vkkpkb466);
  fprintf(pek_save,"%10.12le\n",Vkkpkb666);

 
  return;
  
}

void load_Vkkpkb(void)
{

  int gn,a,b,gi,gj,gk;
  char buf[1], line[200], flyt[20];
  double CI[11];

  for(gj=0; gj<200; gj++)
    line[gj]=' ';

  gi=-1;
  
  
  
  while((buf[0]=getc(pek_save))!='\n')
    {
      if(buf[0]==EOF)
	{vi=vj=-1;return;}
      gi++;

      if(gi>199)
	{printf("Index problem in load_Vkkpkb_new!\n"); exit(0);}

      line[gi]=buf[0];
    }

  gj=0;
  for(gn=0; gn<11; gn++)
    {
      
      for(gk=0; gk<20; gk++)
	flyt[gk]=' ';

      gi=-1;
      while( ((int)line[gj]) != 32 )
	{
	  gi++;

	  if( (gi>19) || (gj> 199) )
	    {printf("Index problem in load_Vkkpkb_new!\n"); exit(0);}
	  
	  flyt[gi]=line[gj];
	  gj++;

	}


      if(gn==0)
	a=atoi(flyt);
      else if(gn==1)
	b=atoi(flyt);
      else
	CI[gn]=atof(flyt);

      gj++;

    }



  Vkkpkb222=CI[2];
  Vkkpkb224=CI[3];
  Vkkpkb244=CI[4];
  Vkkpkb246=CI[5];
  Vkkpkb444=CI[6];
  Vkkpkb446=CI[7];
  Vkkpkb266=CI[8];
  Vkkpkb466=CI[9];
  Vkkpkb666=CI[10];
  /*
  if( (a==i)&&(b==j) ) return;
  else {printf("Problems in load_Vkkpkb!\n"); exit(0);}
  */


  vi=a; vj=b;

}


double POW(int a)
{

  /* (-1)^a */
  
  if(a%2==0)
    return(1.0);
  else
    return(-1.0);

}


/* Rules for three particle operators */
void calc_Vkkpkb(int method, int i, int j)
{
  extern double lval;
  int k,l,m;
  double sum=0.0;

  Vkkpkb222=0.0;
  Vkkpkb224=0.0;
  Vkkpkb244=0.0;
  Vkkpkb246=0.0;
  Vkkpkb444=0.0;
  Vkkpkb446=0.0;
  Vkkpkb266=0.0;
  Vkkpkb466=0.0;
  Vkkpkb666=0.0;

  if((electrons<3)||(lval!=3.0)) return;

  if(method==0)
    {
      for(k=0; k<electrons-2; k++)
	for(l=k+1; l<electrons-1; l++)
	  for(m=l+1; m<electrons; m++)
	    {
	      determinant(Ang[states[i][k]].ml,
			  Ang[states[i][k]].ms, 
			  Ang[states[i][l]].ml, 
			  Ang[states[i][l]].ms, 
			  Ang[states[i][m]].ml,
			  Ang[states[i][m]].ms, 
			  Ang[states[i][k]].ml, 
			  Ang[states[i][k]].ms, 
			  Ang[states[i][l]].ml, 
			  Ang[states[i][l]].ms, 
			  Ang[states[i][m]].ml, 
			  Ang[states[i][m]].ms);
	    }
    }
  
  else if(method==1)
    for(k=0; k<electrons-1; k++)
      for(l=k+1; l<electrons; l++)
	{
	  if(l!=un_paired[1])
	    determinant(Ang[states[i][k]].ml, 
			Ang[states[i][k]].ms, 
			Ang[states[i][l]].ml, 
			Ang[states[i][l]].ms, 
			Ang[states[i][un_paired[1]]].ml, 
			Ang[states[i][un_paired[1]]].ms, 
			
			Ang[states[i][k]].ml, 
			Ang[states[i][k]].ms, 
			Ang[states[i][l]].ml,
			Ang[states[i][l]].ms,
			Ang[states_tmp[un_paired[1]]].ml,
			Ang[states_tmp[un_paired[1]]].ms);
	  
	}

  else if(method==2)
    for(k=0; k<electrons; k++)
      {
	if((k!=un_paired[1]) && (k!=un_paired[2]))
	  determinant(Ang[states[i][k]].ml, 
		      Ang[states[i][k]].ms, 
		      Ang[states[i][un_paired[1]]].ml, 
		      Ang[states[i][un_paired[1]]].ms, 
		      Ang[states[i][un_paired[2]]].ml, 
		      Ang[states[i][un_paired[2]]].ms, 
		      
		      Ang[states[i][k]].ml, 
		      Ang[states[i][k]].ms, 
		      Ang[states_tmp[un_paired[1]]].ml, 
		      Ang[states_tmp[un_paired[1]]].ms, 
		      Ang[states_tmp[un_paired[2]]].ml, 
		      Ang[states_tmp[un_paired[2]]].ms);
	
      }

  else if(method==3)
    determinant(Ang[states[i][un_paired[1]]].ml, 
		Ang[states[i][un_paired[1]]].ms, 
		Ang[states[i][un_paired[2]]].ml, 
		Ang[states[i][un_paired[2]]].ms, 
		Ang[states[i][un_paired[3]]].ml, 
		Ang[states[i][un_paired[3]]].ms, 
		
		Ang[states_tmp[un_paired[1]]].ml, 
		Ang[states_tmp[un_paired[1]]].ms, 
		Ang[states_tmp[un_paired[2]]].ml, 
		Ang[states_tmp[un_paired[2]]].ms, 
		Ang[states_tmp[un_paired[3]]].ml, 
		Ang[states_tmp[un_paired[3]]].ms);
  
  
  else return;


  sum = POW(signe);
  

  Vkkpkb222 = Vkkpkb222 * 1.0 * sum;
  Vkkpkb224 = Vkkpkb224 * 3.0 * sum;
  Vkkpkb244 = Vkkpkb244 * 3.0 * sum;
  Vkkpkb246 = Vkkpkb246 * 6.0 * sum;
  Vkkpkb444 = Vkkpkb444 * 1.0 * sum;
  Vkkpkb446 = Vkkpkb446 * 3.0 * sum;
  Vkkpkb266 = Vkkpkb266 * 3.0 * sum;
  Vkkpkb466 = Vkkpkb466 * 3.0 * sum;
  Vkkpkb666 = Vkkpkb666 * 1.0 * sum; 

   if(fabs(Vkkpkb222)<1e-10) Vkkpkb222=0.0;
  if(fabs(Vkkpkb224)<1e-10) Vkkpkb224=0.0;
  if(fabs(Vkkpkb244)<1e-10) Vkkpkb244=0.0;
  if(fabs(Vkkpkb246)<1e-10) Vkkpkb246=0.0;
  if(fabs(Vkkpkb444)<1e-10) Vkkpkb444=0.0;
  if(fabs(Vkkpkb446)<1e-10) Vkkpkb446=0.0;
  if(fabs(Vkkpkb266)<1e-10) Vkkpkb266=0.0;
  if(fabs(Vkkpkb466)<1e-10) Vkkpkb466=0.0;
  if(fabs(Vkkpkb666)<1e-10) Vkkpkb666=0.0;


}




void determinant(double ml1, double ms1, 
		 double ml2, double ms2, 
		 double ml3, double ms3, 
		 double ml4, double ms4, 
		 double ml5, double ms5, 
		 double ml6, double ms6)
{

  if(ml1-ml4+ml2-ml5+ml3-ml6!=0.0) return;
    
  red_determinant(ml1, ms1, ml4, ms4,
		  ml2, ms2, ml5, ms5,
		  ml3, ms3, ml6, ms6, 1.0);
  
  red_determinant(ml1, ms1, ml4, ms4,
		  ml2, ms2, ml6, ms6,
		  ml3, ms3, ml5, ms5, -1.0);
  
  red_determinant(ml1, ms1, ml5, ms5,
		  ml2, ms2, ml6, ms6,
		  ml3, ms3, ml4, ms4, 1.0);
  
  red_determinant(ml1, ms1, ml5, ms5,
		  ml2, ms2, ml4, ms4,
		  ml3, ms3, ml6, ms6, -1.0);
  
  red_determinant(ml1, ms1, ml6, ms6,
		  ml2, ms2, ml4, ms4,
		  ml3, ms3, ml5, ms5, 1.0);
  
  red_determinant(ml1, ms1, ml6, ms6,
		  ml2, ms2, ml5, ms5,
		  ml3, ms3, ml4, ms4, -1.0);
     
} 


void red_determinant(double ml1, double ms1, 
		     double ml1p, double ms1p, 
		     double ml2, double ms2, 
		     double ml2p, double ms2p, 
		     double ml3, double ms3, 
		     double ml3p, double ms3p,
		     double factor)
{

  if((ms1!=ms1p)||(ms2!=ms2p)||(ms3!=ms3p))
    return;


  // ======================================================= 
  Vkkpkb222+= factor * 6.0 * triple_scalar(2.0, 2.0, 2.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  // ======================================================= 

  Vkkpkb224+= factor * 2.0 * triple_scalar(2.0, 2.0, 4.0,  
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb224+= factor * 2.0 * triple_scalar(2.0, 4.0, 2.0,
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);  
  Vkkpkb224+= factor * 2.0 * triple_scalar(4.0, 2.0, 2.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p); 

  // ======================================================= 

  Vkkpkb244+= factor * 2.0 * triple_scalar(2.0, 4.0, 4.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb244+= factor * 2.0 * triple_scalar(4.0, 2.0, 4.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb244+= factor * 2.0 *triple_scalar(4.0, 4.0, 2.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  
 // ======================================================= 
				     
  Vkkpkb246+= factor * triple_scalar(2.0, 4.0, 6.0, 
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);

  Vkkpkb246+= factor * triple_scalar(2.0, 6.0, 4.0, 
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);

  Vkkpkb246+= factor * triple_scalar(4.0, 2.0, 6.0, 
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);

  Vkkpkb246+= factor * triple_scalar(4.0, 6.0, 2.0, 
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);

  Vkkpkb246+= factor * triple_scalar(6.0, 2.0, 4.0,
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);

  Vkkpkb246+= factor * triple_scalar(6.0, 4.0, 2.0, 
				     ml1, ms1, ml1p, ms1p,
				     ml2, ms2, ml2p, ms2p,
				     ml3, ms3, ml3p, ms3p);
  

  // ======================================================= 

  Vkkpkb444+= factor * 6.0 * triple_scalar(4.0, 4.0, 4.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  
  // ======================================================= 

  Vkkpkb446+= factor * 2.0 * triple_scalar(4.0, 4.0, 6.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb446+= factor * 2.0 * triple_scalar(4.0, 6.0, 4.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb446+= factor * 2.0 * triple_scalar(6.0, 4.0, 4.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);

  // ======================================================= 

  Vkkpkb266+= factor * 2.0 * triple_scalar(2.0, 6.0, 6.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb266+= factor * 2.0 * triple_scalar(6.0, 2.0, 6.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb266+= factor * 2.0 * triple_scalar(6.0, 6.0, 2.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);

  // ======================================================= 

  Vkkpkb466+= factor * 2.0 * triple_scalar(4.0, 6.0, 6.0,
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb466+= factor * 2.0 * triple_scalar(6.0, 4.0, 6.0,
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  Vkkpkb466+= factor * 2.0 * triple_scalar(6.0, 6.0, 4.0,
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
  // ======================================================= 

  Vkkpkb666+= factor * 6.0 * triple_scalar(6.0, 6.0, 6.0, 
					   ml1, ms1, ml1p, ms1p,
					   ml2, ms2, ml2p, ms2p,
					   ml3, ms3, ml3p, ms3p);
}
 
double triple_scalar(double k1, double k2, double k3,
		     double ml1, double ms1, 
		     double ml1p, double ms1p, 
		     double ml2, double ms2, 
		     double ml2p, double ms2p, 
		     double ml3, double ms3, 
		     double ml3p, double ms3p)
{
  
  double q1, q2, q3, t1, t2, t3, t4;
  
  q1 = ml1 - ml1p;
  q2 = ml2 - ml2p;
  q3 = ml3 - ml3p;
  
  
  
  t1 = clebsg( k1, k2, k3, q1, q2, q3, 0, 0, 0, 1);

  if(t1==0.0)
    return(0.0);
  
  
  t2 = clebsg( lval, k1, lval, -ml1, q1, ml1p, 0, 0, 0, 1);
  t3 = clebsg( lval, k2, lval, -ml2, q2, ml2p, 0, 0, 0, 1);
  t4 = clebsg( lval, k3, lval, -ml3, q3, ml3p, 0, 0, 0, 1);

  return(t1*t2*t3*t4*POW((int)(3.0*lval-ml1-ml2-ml3))
	 *sqrt((2.0*k1+1.0)*(2.0*k2+1.0)*(2.0*k3+1.0)));

}






