/* two_part.c */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <stdlib.h>
#include <math.h>

void precalc_twopart(void);

void two_particle(int i, int j, int method, int ii);
void Rtwo_particle(int i, int j, int method, int ii);
void red_two(int i, int j, int k, int l);

void Matrix_es(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);

void Matrix_alpha(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);

void Matrix_beta(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);

void Matrix_gamma(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);

void Matrix_mk(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);

void Matrix_pk(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);


double ck(double k, double mla, double mlb);
double Rk(double k);
extern double Delta(double a, double b);
extern double POW(int a);
double ukuk(double power, double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd);



double ww0(double ml1p, double ms1p, double ml2p, double ms2p, 
	   double ml1, double ms1, double ml2, double ms2, 
	   double kappa1, double kappa2, double k1, double k2, double k);



double uutt0(double ml1p, double ms1p, double ml2p, double ms2p, 
	     double ml1, double ms1, double ml2, double ms2, 
	     double K1, double K2, double k1, double k2, double k);
typedef double doublereal;
typedef struct { doublereal r, i; } doublecomplex;
typedef long int integer;

struct state {
double ml;
double ms;
};


extern double *pre2;
extern double RE0, IM0, RE1, IM1, lval;
extern struct state *Ang;
extern int electrons;
extern int **states, *states_tmp, signe, *un_paired;
extern doublecomplex *AP;
extern doublereal *rAP;



extern double F2, F4, F6, Xi, Alpha, Beta, Gamma;
extern double Mk[10], Pk[10];

extern double clebsg(double a, double b, double c, double xx, double yy, double zz, double gg, double hh, double pp, int mode);


void two_particle(int i, int j, int method, int ii)
{
  int k,l;

  RE0 = IM0 = 0.0;
 
  if(method==0)
    {
      for(k=0; k<electrons-1; k++)
	{
	  for(l=k+1; l<electrons; l++)
	    {
	      red_two(states[i][l], states[i][k],
		      states[i][l], states[i][k]);

	      RE0 += RE1;
	      IM0 += IM1;
	      

	      red_two(states[i][l], states[i][k], 
		      states[i][k], states[i][l]);
	      
	      RE0 -= RE1;
	      IM0 -= IM1;

	    }
	}

      AP[ii].r += RE0;
      AP[ii].i += IM0;

    }

  else if(method==1)
    {
      
      for(k=0; k<electrons; k++)
	{
	  red_two(states[i][k], states[i][un_paired[1]], 
		  states[i][k], states_tmp[un_paired[1]]);

	  RE0 += RE1;
	  IM0 += IM1;    
	  
	  red_two(states[i][k], states[i][un_paired[1]], 
		  states_tmp[un_paired[1]], states[i][k]);
	  
	  RE0 -= RE1;
	  IM0 -= IM1;
	  
	}

      AP[ii].r += RE0*pow(-1.0,(double)signe);
      AP[ii].i += IM0*pow(-1.0,(double)signe);
      
    }

  else if(method==2)
    {

      red_two(states[i][un_paired[1]], states[i][un_paired[2]],
	      states_tmp[un_paired[1]], states_tmp[un_paired[2]]);
      RE0 += RE1;
      IM0 += IM1;    

      red_two(states[i][un_paired[1]], states[i][un_paired[2]],
	      states_tmp[un_paired[2]], states_tmp[un_paired[1]]);  
      
      RE0 -= RE1;
      IM0 -= IM1;
      
      AP[ii].r += RE0*pow(-1.0,(double)signe);
      AP[ii].i += IM0*pow(-1.0,(double)signe);

    }

}

void Rtwo_particle(int i, int j, int method, int ii)
{
  int k,l;

  RE0 = 0.0;
 
  if(method==0)
    {
      for(k=0; k<electrons-1; k++)
	{
	  for(l=k+1; l<electrons; l++)
	    {
	      red_two(states[i][l], states[i][k],
		      states[i][l], states[i][k]);

	      RE0 += RE1;
	      

	      red_two(states[i][l], states[i][k], 
		      states[i][k], states[i][l]);
	      
	      RE0 -= RE1;

	    }
	}

      rAP[ii] += RE0;

    }

  else if(method==1)
    {
      
      for(k=0; k<electrons; k++)
	{
	  red_two(states[i][k], states[i][un_paired[1]], 
		  states[i][k], states_tmp[un_paired[1]]);

	  RE0 += RE1;
	  
	  red_two(states[i][k], states[i][un_paired[1]], 
		  states_tmp[un_paired[1]], states[i][k]);
	  
	  RE0 -= RE1;
	  
	}

/*            if((RE0!=0.0)||(IM0!=0.0)) */
/* 	      printf("hoppla\n"); */

	
      rAP[ii] += RE0*pow(-1.0,(double)signe);

      
    }

  else if(method==2)
    {

      red_two(states[i][un_paired[1]], states[i][un_paired[2]],
	      states_tmp[un_paired[1]], states_tmp[un_paired[2]]);
      RE0 += RE1;

      red_two(states[i][un_paired[1]], states[i][un_paired[2]],
	      states_tmp[un_paired[2]], states_tmp[un_paired[1]]);  
      
      RE0 -= RE1;
      
      rAP[ii] += RE0*pow(-1.0,(double)signe);

    }

}


void red_two(int i, int j, int k, int l)
{
  double mla, msa, mlb, msb;
  double mlc, msc, mld, msd;
  int n3,n2,n1,len;

  len = 2*(2*(int)lval+1);
  n1=len; n2=n1*len; n3=n2*len;

 
  RE1=IM1=0.0;

  /*
  mla = Ang[i].ml;     mlc = Ang[k].ml;
  msa = Ang[i].ms;     msc = Ang[k].ms;
  mlb = Ang[j].ml;     mld = Ang[l].ml;
  msb = Ang[j].ms;     msd = Ang[l].ms;

  
  
  Matrix_es( mla, msa, mlb, msb, mlc, msc, mld, msd); 
  Matrix_alpha( mla, msa, mlb, msb, mlc, msc, mld, msd); 
  Matrix_beta( mla, msa, mlb, msb, mlc, msc, mld, msd); 
  Matrix_gamma( mla, msa, mlb, msb, mlc, msc, mld, msd); 
  Matrix_mk( mla, msa, mlb, msb, mlc, msc, mld, msd); 
  Matrix_pk( mla, msa, mlb, msb, mlc, msc, mld, msd); 

  printf("%d %d %d %d %d \n",i,j,k,l,i*n3 + j*n2 + k*n1 + l);
  printf("%e\n",RE1);
  */
  RE1 = pre2[i*n3 + j*n2 + k*n1 + l];IM1=0.0;
  
  //printf("%e\n",RE1);getchar();


}




void Matrix_es(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{

  double sum=0;
  int k;

  if(msa!=msc) return;
  if(msb!=msd) return;
  if(mla+mlb!=mlc+mld) return;

  /* k=0 is excluded since it's a central contribution equal for all levels */

  for(k=2; k<=2*(int)lval; k=k+2)
    sum = sum + ck((double)k,mla,mlc)
              * ck((double)k,mld,mlb)
              * Rk((double)k);

  RE1 += sum;

}



double Rk(double k)
{

  if(k==2)
    return(F2);
  else if(k==4)
    return(F4);
  else if(k==6)
    return(F6);
  else 
    { printf("Problem in Rk!\n"); exit(0); }
}


double ck(double k, double mla, double mlb)
{

  double trej1,trej2;

  trej1 = clebsg( lval, k, lval, 0, 0, 0,0, 0, 0, 1);
  trej2 = clebsg( lval, k, lval, -mla, mla-mlb, mlb, 0, 0, 0, 1);

  return(pow(-1.0,mla)*(2.0*lval+1.0)*trej1*trej2);

}



void Matrix_alpha(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{

  double C;
  
  C = 2.0*lval*(lval+1.0)*(2.0*lval+1.0);


  RE1 += Alpha * C * ukuk(1.0, mla, msa, mlb, msb, mlc, msc, mld, msd); 


}





void Matrix_beta(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{

  RE1 += Beta  * (1.5* ukuk(1.0, mla, msa, mlb, msb, 
			    mlc, msc, mld, msd) +
		  5.5* ukuk(5.0, mla, msa, mlb, msb, 
			    mlc, msc, mld, msd));


}

void Matrix_gamma(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{

  double sum=0.0, power;
  int t;


  for(t=1; t<=(int)lval; t++)
    {
      power = 2.0*(double)t - 1.0;
      sum = sum + (2.0*power+1.0) * 
	ukuk(power, mla, msa, mlb, msb, 
	            mlc, msc, mld, msd);
    }

  RE1 += Gamma*(2.0/5.0)*sum;
}



double ukuk(double power, double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{

  double q;
  double trej1,trej2;

  if(mla-mlc != mld-mlb) 
    return(0.0);
  
  if((msa !=msc) || (msb != msd)) 
    return(0.0);

  q=mla-mlc;

  trej1 = clebsg( lval, power, lval, -mla, q, mlc,0, 0, 0, 1);
  trej2 = clebsg( lval, power, lval, -mlb, -q, mld, 0, 0, 0, 1);

  return(pow(-1,mla+mld)*trej1*trej2);

}





double ww0(double ml1p, double ms1p, double ml2p, double ms2p, 
	   double ml1, double ms1, double ml2, double ms2, 
	   double K1, double K2, double k1, double k2, double k)
{

/*====================================
    (psip | (w w)(kk)0 | psi )

    w = t^kappa u^k
=====================================*/

  double t1, t2, t3, t4, t5, t6, s = 0.5;
  double qp, fas, Q1, Q2, q1, q2;

  Q1 = ms1p-ms1;
  Q2 = ms2p-ms2;
  q1 = ml1p-ml1;
  q2 = ml2p-ml2;
  qp = ms1p-ms1+ms2p-ms2;

  t1 = clebsg( s,  K1, s, -ms1p, Q1, ms1, 0,0,0,1);
  t2 = clebsg( s,  K2, s, -ms2p, Q2, ms2, 0,0,0,1);
  t3 = clebsg( K1, k, K2, -Q1, qp, -Q2, 0,0,0,1);
  
  
  t4 = clebsg( lval,  k1, lval, -ml1p, q1, ml1, 0,0,0,1);
  t5 = clebsg( lval,  k2, lval, -ml2p, q2, ml2, 0,0,0,1);
  t6 = clebsg( k1, k, k2, -q1, -qp, -q2, 0,0,0,1);


  
  fas = POW((int)( (s-ms1p) + (s-ms2p) + (lval-ml1p) + (lval-ml2p) + 
		   (K1-Q1)  + (K2-Q2)  + (k1-q1)  + (k2-q2)  + (k-qp)));

  return( fas * sqrt( (2.0*K1+1.0) * (2.0*K2+1.0) *
		(2.0*k1+1.0) * (2.0*k2+1.0) * (2.0*k+1.0)) *
	  t1*t2*t3*t4*t5*t6);

}

double mkss(double ml1p, double ms1p, double ml2p, double ms2p, 
	    double ml1, double ms1, double ml2, double ms2, 
	    double k)
{

  double sum, red1, red2;
  
  red1 = clebsg( lval, k, lval, 0.0, 0.0, 0.0, 0, 0, 0, 1) * POW((int)lval)
    * (2.0*lval+1.0);
  red2 = clebsg( lval, k+2.0, lval, 0.0, 0.0, 0.0, 0, 0, 0, 1) * POW((int)lval)
    * (2.0*lval+1.0);
  
   
   sum = 
     ww0( ml1p, ms1p, ml2p, ms2p, ml1,  ms1,  ml2,  ms2, 
	  1.0, 1.0, k, k+2.0, 2.0)
     +
     ww0( ml1p, ms1p, ml2p, ms2p, ml1,  ms1,  ml2,  ms2, 
	  1.0, 1.0, k+2.0, k, 2.0);

   
   sum = sum * (-2.0*sqrt((k+1.0)*(k+2.0)*(2.0*k+3.0))) * red1 * red2;
   /*
   printf("%e %e\n",ww0( ml1p, ms1p, ml2p, ms2p, ml1,  ms1,  ml2,  ms2, 
	  1.0, 1.0, k, k+2, 2.0),ww0( ml1p, ms1p, ml2p, ms2p, ml1,  ms1,  ml2,  ms2, 
	  1.0, 1.0, k+2, k, 2.0));
   getchar();
   */
   return(sum);
}






void Matrix_mk(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{
  double red1, red2, m1, m2, m1p, m2p;
  double fac;
  int k,K;
  /* A = mkss(mla, msa, mlb, msb,  */
  /* 	   mlc, msc, mld, msd, 2.0); */
  
  if(msa-msc+msb-msd!=-(mla-mlc+mlb-mld)) return;


  for(k=0;k<=2*(int)lval-2;k=k+2)
    RE1 +=  Mk[k] * mkss(mla, msa, mlb, msb, mlc, msc, mld, msd, (double)k);

  
  for(k=0;k<=2*(int)lval-1;k++)
    {
      K = (double)k;

      m1 = ww0(mla, msa, mlb, msb, mlc, msc, mld, msd,
	       0.0, 1.0, K+1.0, K, 1.0);
      m1p = ww0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		1.0, 0.0, K, K+1.0, 1.0);
      m2 = ww0(mla, msa, mlb, msb, mlc, msc, mld, msd,
	       0.0, 1.0, K, K+1.0, 1.0);
      m2p = ww0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		1.0, 0.0, K+1.0, K, 1.0);

      fac = sqrt( (K+1.0)*(2.0*lval+K+2)*(2.0*lval-K));
      
      if(k%2==1)
	{
	  red1 = clebsg( lval, K+1.0, lval,0.0,0.0,0.0,0,0,0,1) * 
	    (2.0*lval+1.0); red1=red1*red1;
	  RE1 += Mk[k-1] * fac * red1 * ( m1 + m1p + 2.0 * m2 + 2.0 * m2p);
	  
	}
      else
	{
	  red2 = clebsg( lval, K, lval,0.0,0.0,0.0,0,0,0,1) * 
	    (2.0*lval+1.0); red2=red2*red2;

	  RE1 += Mk[k] * fac * red2 * (2.0 * m1 + 2.0 * m1p + m2 + m2p);
	}
    }
  
  
  /*
  S=1.0; L = 1.0;
  Sp=1.0; Lp = 3.0;
  J = 2.0; t=2.0;
  printf("%e\n",clebsg(Sp, Lp, J, L, S, t, 0,0,0,2)*POW((int)(Sp+L+J)));
  exit(0);
  */
}



double uutt0(double ml1p, double ms1p, double ml2p, double ms2p, 
	     double ml1, double ms1, double ml2, double ms2, 
	     double K1, double K2, double k1, double k2, double k)
{


 double t1, t2, t3, t4, t5, t6, s = 0.5;
  double qp, fas, Q1, Q2, q1, q2;

  Q1 = ms1p-ms1;
  Q2 = ms2p-ms2;
  q1 = ml1p-ml1;
  q2 = ml2p-ml2;
  qp = ms1p-ms1+ms2p-ms2;

  t1 = clebsg( s,  K1, s, -ms1p, Q1, ms1, 0,0,0,1);
  t2 = clebsg( s,  K2, s, -ms2p, Q2, ms2, 0,0,0,1);
  t3 = clebsg( K1, k, K2, -Q1, qp, -Q2, 0,0,0,1);
  
  
  t4 = clebsg( lval,  k1, lval, -ml1p, q1, ml1, 0,0,0,1);
  t5 = clebsg( lval,  k2, lval, -ml2p, q2, ml2, 0,0,0,1);
  t6 = clebsg( k1, k, k2, -q1, -qp, -q2, 0,0,0,1);


  
  fas = POW((int)( k+(s-ms1p) + (s-ms2p) + (lval-ml1p) + (lval-ml2p) + 
		   (K1-Q1)  + (K2-Q2)  + (k1-q1)  + (k2-q2)  + (k-qp)));

  return( fas * (2.0*k+1.0) * t1*t2*t3*t4*t5*t6);
  
}


void Matrix_pk(double mla, double msa, double mlb, double msb, double mlc, double msc, double mld, double msd)
{
  
  double red1, m1, m2, m1p, m2p;
  double fac1, fac2,K;
  int k;


  
  if(msa-msc+msb-msd!=-(mla-mlc+mlb-mld)) return;



  for(k=2;k<=2*(int)lval;k=k+2)
    {
      K = (double)k;
      
      m1 = uutt0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		 0.0, 1.0, K, K-1.0, 1.0);
      m1p = uutt0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		  1.0, 0.0, K-1.0, K, 1.0);
      m2 = uutt0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		 0.0, 1.0, K, K+1.0, 1.0);
      m2p = uutt0(mla, msa, mlb, msb, mlc, msc, mld, msd,
		  1.0, 0.0, K+1.0, K, 1.0);
  
      red1 = clebsg( lval, K, lval,0.0,0.0,0.0,0,0,0,1) * 
	(2.0*lval+1.0); red1=red1*red1;

      fac1 = sqrt( (2.0*lval+K+1.0) * (2.0*lval-K+1.0) *
		   K * (2.0*K-1.0) / (2.0*K+1.0));
      fac2 = sqrt( (2.0*lval+K+2.0) * (2.0*lval-K) * 
		   (K+1.0) * (2.0*K+3.0) / (2.0*K+1.0));

  
      RE1 += (Pk[k]/6.0) * red1 * (fac1 * (m1 + m1p)  -  fac2 * (m2 + m2p));
      
    }
}


void precalc_twopart(void)
{
  int i,j,k,l,n1,n2,n3,len,I,J;
  double mla,mlb,mlc,mld,msa,msb,msc,msd;

  
  len=2*(2*(int)lval+1);
  n1=len; n2=n1*len; n3=n2*len;
  
  if((electrons<1)||(electrons>12)) {
    for(i=0;i<n3*len;i++) 
      pre2[i]=0.0;
    
    return;
  }

  /*
  printf("kk\n");
  for(i=0;i<len;i++) {
    for(j=0;j<len;j++) {

      if(i!=j) {
	mla = Ang[i].ml;
	msa = Ang[i].ms;  
	mlb = Ang[j].ml; 
	msb = Ang[j].ms;
      }

      for(k=0;k<len;k++) {
	for(l=0;l<len;l++) {

	  RE1=IM1=0.0;
	  
	  if(k!=l&&i!=j) {
	    mlc = Ang[k].ml;
	    msc = Ang[k].ms;
	    mld = Ang[l].ml;
	    msd = Ang[l].ms;
	    
	    
	    if((mla+mlb+msa+msb==mlc+mld+msc+msd)) {
	    Matrix_es( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    Matrix_alpha( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    Matrix_beta( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    Matrix_gamma( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    Matrix_mk( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    Matrix_pk( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    }
	  }
	  
	  pre2[i*n3 + j*n2 + k*n1 + l]=RE1;
	  
  
	}
      }
    }
  }
  */

  
  for(i=0;i<len;i++) {
    for(j=0;j<len;j++) {
      I = i*n1+j;
      mla = Ang[i].ml;
      msa = Ang[i].ms;  
      mlb = Ang[j].ml; 
      msb = Ang[j].ms;

      for(k=0;k<len;k++) {
	for(l=0;l<len;l++) {

	  J=k*n1+l;
	  

	  if(I<=J) {
	    RE1=IM1=0.0;
	    mlc = Ang[k].ml;
	    msc = Ang[k].ms;
	    mld = Ang[l].ml;
	    msd = Ang[l].ms;
	    
	    
	    if((mla+mlb+msa+msb==mlc+mld+msc+msd)) {
	      Matrix_es( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	      Matrix_alpha( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	      Matrix_beta( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	      Matrix_gamma( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	      Matrix_mk( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	      Matrix_pk( mla, msa, mlb, msb, mlc, msc, mld, msd); 
	    }
	    pre2[i*n3 + j*n2 + k*n1 + l] = RE1;
	    pre2[k*n3 + l*n2 + i*n1 + j] = RE1;
	  }
	  
	    
	  
  
	}
      }
    }
  }
  

}

