program freeET
! free energy of magnetic system F, M= -dF/dB, chi=-d2F/dB2
! dependence of susceptibility and magnetic moment on temperature 
! for selected field B. Energy must be calculated for 5 fields
! b-2db, b-db, b, b+db, b+2db using program REcfp 
! step dB of B must be small enough to allow derivative d2F/dB2 
! July 15. 2016. 
implicit real*8(a-h,o-z)
REAL*8,ALLOCATABLE  :: magt(:)
REAL*8,ALLOCATABLE  :: e(:,:),f(:)
read(5,*)nstates,b,db,nt,t0,dt,ipr
write(6,*)' Temperature dependence of M and chi from free energy'
write(6,106)nstates,nt,b
106 format(' nstates=',i4,'  ntemp.=',i4,' B=',f7.2,' [T]')
write(6,105)db
105 format(' step of B for derivative=',f6.3,' [T]')
nb=5               ! to calculate d2F/dB2
cfKmeV=0.08616     ! K -> meV
cf1=17.276         ! E[meV] / mu_B
cfchi=0.5585       ! mu_B/T -> emu/mol
cfcmK=1.4388       ! cm-1 -> K
allocate (e(nstates,nb),f(nb),magt(nb))
do ib=1,nb
 do istates=1,nstates   
  read(66,*)kstates,e(istates,ib)        !  e in cm-1, output of REcfp
  if(ipr.gt.1)write(6,104)ib,istates,kstates,e(istates,ib)
104 format(3i5,f16.4,' ib,istates,kstates,e')
 enddo
enddo
e0=e(1,1)
if(ipr.gt.1)write(6,*)' state  iB      E [cm-1]         E [K]'
do istates=1,nstates
 do ib=1,nb
  e(istates,ib)=cfcmK*(e(istates,ib)-e0) !  e in K, relative to e0
  if(ipr.gt.1)then
   eorig=e0+e(istates,ib)/cfcmK
   write(6,101)istates,ib,eorig,e(istates,ib)
  endif
 enddo
enddo

write(6,*)
write(6,*)'  T[K]       freeE[K]     m[muB]     chi[emu/mol]   1/chi[mol/emu]'
t=t0
do it=1,nt       ! loop over temperature
 do ib=1,nb
  z=0.               ! sum of states
  do istates = 1,nstates
   pop=exp(-e(istates,ib)/t)
   z=z+pop
  enddo
  f(ib)=-t*dlog(z)   ! free energy, F=-kT.ln(z) [K]
  if(ipr.gt.1)write(6,102)ib,t,f(ib)
 enddo
 do ib=2,nb-1            
  d1f=(f(ib+1)-f(ib-1))/(2.*db)  ! first derivative of f
  magt(ib)=-cf1*cfKmeV*d1f    ! moment in mu_B, m = -dF/dB
  if(ipr.gt.1)write(6,103)ib,T,magt(ib)
 enddo
 do ib=3,nb-2            
  d1m=(magt(ib+1)-magt(ib-1))/(2.*db)  ! derivative of moment
 enddo                                     
 chit=cfchi*d1m  ! chi in emu/mol, chi=-d2F/dB2= dm/dB 
 chiinv=1./chit
 write(6,100)T,f(3),magt(3),chit,chiinv
 write(24,100)T,f(3),magt(3),chit,chiinv
 t=t+dt
enddo
100 format(f9.3,2f12.4,f14.6,f14.3)
101 format(2i5,2f16.6)
102 format(i5,f9.3,f14.6,' ib, T, F(ib)')
103 format(i5,f9.3,f14.6,' ib, T, m(ib)')
stop
end
