program highT 
! temperature dependence of susceptibility and magnetic moment 
! for fixed magnetic field b 
! assuming magnetic energy << thermal energy
!  December 14. 2016. 
implicit real*8(a-h,o-z)
real*8 magt(3),chit(3),ft(3)
REAL*8,ALLOCATABLE  :: e0(:),g(:,:),chi(:,:),mag(:)
read(5,*)nstate,ncomp,nt,t0,dt,b,db
write(6,201)nstate,ncomp
201 format(' number of states=',i4,', number of components',i4)
203 format(' step for chi calculation dB=',f7.3,' T')
write(6,205)nt,t0,dt,b
write(6,203)db
205 format(' number of T',i4,', T0=',f7.3,', dT=',f7.3,', B=',f7.2,' T')
write(6,*)
allocate (e0(nstate),g(ncomp,nstate),chi(ncomp,nstate),mag(nstate))

cf1=17.276         ! E[meV] / mu_B
cft=0.08616        ! K -> meV

 do istate=1,nstate
  read(36,*)k1,emev,(g(j,istate),j=1,ncomp)    ! fort.36 is output of regr
  read(36,*)k1,emev,(chi(j,istate),j=1,ncomp)
  e0(istate)=abs(emev)/cft  ! all e0 must be nonnegative, meV -> K
 enddo
100 format(i6,f9.2,3f8.3,3f10.5)
write(6,*)
if(ncomp.eq.1)then
  write(6,*) '  T[K]     Fx      Mx      CHix     '
 elseif(ncomp.eq.2)then
  write(6,*)'  T[K]     Fx      Fy      Mx        My        CHix       CHiy'
 elseif(ncomp.eq.3)then
  write(6,*) &       
  '  T[K]     Fx      Fy     Fz      Mx        My        Mz         CHix        CHiy        CHiz'
  write(9,*)' Averaged susceptibility in polycrystals'
  write(9,*)'  T[K]    chiCW       chivV    averaged chi'
endif
emax=-1000.
t=t0
do it=1,nt       ! loop over temperature
 do icomp=1,ncomp    ! loop over axes
  z0=0.
  z1=0.
  z2=0.          ! sum of states
  z22=0.
  tmag=0.       ! M(T)
  do istate = 1,nstate
   gi=g(icomp,istate)
   chii=chi(icomp,istate)
   emag=(gi*b - chii*b**2)/(2.*cf1)         ! Emagnetic in meV
   emag=emag/cft
   if(emag.gt.emax)then
    emax=emag     
    imax=istate
   endif
   pop=exp(-e0(istate)/t)
   z0=z0+pop
   z1=z1+pop*gi
   z2=z2+pop*chii
   z22=z22+pop*gi*gi
  enddo                           ! end nstate
  write(9,108)icomp,imax,emax,t
108 format('component',i4,', state',i4,', magn. energy',f10.4,', kT',f10.4)
!  ft(icomp)=-t*ln(z0) - b*z1/(2.*z0) + b*b*z2/(2.*z0) + b*b*z22/(8.*t*z0)
  ft(icomp)= - b*z1/(2.*z0) + b*b*z2/(2.*z0) + b*b*z22/(8.*t*z0)
  magt(icomp)=z1/(2.*z0) - b*z2/z0 - (b*z22)/(4.*t*z0)
  chit(icomp)=z2/z0 + z22/(4.*t*z0)
 enddo           ! end comp
 if(ncomp.eq.1)then
  write(6,101)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
  write(40,101)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
 elseif(ncomp.eq.2)then
  write(6,102)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
  write(40,102)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
 elseif(ncomp.eq.3)then
  write(6,103)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
  write(40,103)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)

! calculate averaged susceptibility in polycrystals
  chiCW=0.
  chivV=0.
  do icomp=1,ncomp
   chiCW=chiCW+z22/(4.*t*z0)
   chivV=chivV+z2/z0
  enddo
  chiav=chiCW+chivV
  write(9,109)t,chiCW,chivV,chiav

 elseif(ncomp.eq.4)then
  write(6,104)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
  write(40,104)t, &
   (ft(icomp),icomp=1,ncomp),(abs(magt(icomp)),icomp=1,ncomp),(chit(icomp),icomp=1,ncomp)
 endif

 t=t+dt
enddo
101 format(f7.1,f8.2,f10.4,f12.6)
102 format(f7.1,2f8.2,2f10.5,2f12.6)
103 format(f7.1,3f8.2,3f10.5,3f12.6)
104 format(f7.1,4f8.2,4f10.5,4f12.6)
109 format(f7.1,3f12.6)
stop
end
