program tempav 
! temperature dependence of susceptibility 
! and magnetic moment  
! May 6.2013. 
! susceptibility is calculated as [m(B)-m(B-db)]/db
implicit real*8(a-h,o-z)
real*8 magphi,magaver
REAL*8,ALLOCATABLE  :: e0(:),g(:,:),chi(:,:),mag(:),magb(:)
read(5,*)kramers,nstate
write(6,201)kramers,nstate
201 format(' Kramers=',i3,', number of states=',i4)
read(5,*)nphi,nx
write(6,202)nphi,nx
202 format(' averaging over',i6,' phi points and',i6,' cos(theta) points')
!read(5,*)nb,b0,db,dbb
nb=1
db=0.
read(5,*)nt,t0,dt,b0,dbb
write(6,204)dbb
204 format(' step for calculating derivative dm/dB =',f7.4,' T')
write(6,205)nt,t0,dt
205 format(' number of T',i4,', T0=',f7.3,', dT=',f7.3)
write(6,*)
allocate (e0(nstate),g(3,nstate),chi(3,nstate),mag(nstate),magb(3))

cf=0.12398         ! cm-1 -> meV
cf1=-17.276         ! E[meV] / mu_B
cft=0.08616        ! K -> meV
pi=3.141592653589793
dx=1./(nx-1)
dphi=pi/(nphi-1)

if(kramers.eq.1)then
 write(6,*)' Kramers ion, E(B=0) [meV], g, chi in canonical form'
 write(6,*)' doublet   E0     gx      gy     gz      chix     chiy      chiz'
 k=0
 do istate=1,nstate,2
  k=k+1
  read(36,*)k1,e0(istate),(g(j,istate),j=1,3)
  read(36,*)k1,e0(istate),(chi(j,istate),j=1,3)
  write(6,100)k,e0(istate),(g(j,istate),j=1,3),(chi(j,istate),j=1,3)
  e0(istate+1)=e0(istate)
  do j=1,3
   g(j,istate+1)=g(j,istate)
   chi(j,istate+1)=chi(j,istate)
  enddo
 enddo
else
 write(6,*)' non Kramers ion, E(B=0) [meV], g, chi in canonical form'
 write(6,*)' singlet   E0     gx      gy     gz      chix     chiy      chiz'
 do istate=1,nstate
  read(79,*)k1,e0(istate),(g(j,istate),j=1,3)
  read(79,*)k1,e0(istate),(chi(j,istate),j=1,3)
  write(6,100)istate,e0(istate),(g(j,istate),j=1,3),(chi(j,istate),j=1,3)
  e0(istate)=e0(istate)/cft             ! meV -> K
 enddo
endif
100 format(i6,f9.2,3f8.3,3f10.5)
write(6,*)
write(6,*)'   T [K]   Bext [T]   <M> [muB]    chi [muB/T**2]'
t=t0
do it=1,nt       ! loop over temperature
 b=b0
 do ib=1,nb      ! loop over magnetic fields
  bd=b-dbb      ! field for derivative
  do ibb=1,3
   x=0.            ! cos(theta) -> x
   magaver=0.
   do ix=1,nx      ! loop over angle theta
    phi=0.
    magphi=0.
    do iphi=1,nphi ! loop over angle phi
     c2=cos(phi)**2
     s2=1.-c2
     z=0.          ! sum of states
     tmag=0.       ! M(T)
     sgkram=-1.
     do istate = 1,nstate
      g2=(1-x*x)*(c2*g(1,istate)**2+s2*g(2,istate)**2)+x**2*g(3,istate)**2
      chi2=(1-x*x)*(c2*chi(1,istate)**2+s2*chi(2,istate)**2)+x**2*chi(3,istate)**2
      gi=0.
      if(g2.gt.0.)gi=sqrt(g2)
      chii=0.
      if(chi2.gt.0.)chii=sqrt(chi2)
      if(kramers.eq.1)then
       e=e0(istate) + sgkram*gi*bd/2. - chii*bd**2/2.
       mag(istate)=sgkram*gi/2.-chii*bd
       sgkram=-sgkram
      else
       e=e0(istate) + gi*bd/2. - chii*bd**2/2.
       mag(istate)=gi/2.-chii*bd
      endif
      pop=exp(-e/t)
      z=z+pop
      tmag=tmag+pop*mag(istate)
if((ix+iphi).eq.2)then
!if(istate.lt.3)write(99,222)it,ib,ibb,t,b,bd,e,pop
endif
222 format(3i5,5f10.4)
     enddo
     tmag=tmag/z
     if((iphi.eq.1).or.(iphi.eq.nphi))then
      magphi=magphi+tmag/2.
     else
      magphi=magphi+tmag
     endif
     phi=phi+dphi
if((ix+iphi).eq.2)then
!write(98,222)it,ib,ibb,t,b,bd,tmag,magphi
!if(ibb.eq.2)write(90+ib,102)t,magphi,b
endif
    enddo
    magphi=dphi*magphi/pi
    if((ix.eq.1).or.(ix.eq.nx))then
     magaver=magaver+magphi/2.
    else
     magaver=magaver+magphi
    endif
    x=x+dx
   enddo
   magaver=magaver*dx
   magb(ibb)=-magaver       ! change sign so that m>0
   bd=bd+dbb
  enddo
  chiaver=(magb(3)-magb(1))/(2.*dbb)
  write(6,101)t,b,magb(2),chiaver
  write(40+ib,102)t,magb(2),chiaver,b
  b=b+db
 enddo
 t=t+dt
enddo
101 format(2f8.2,f12.3,f12.6)
102 format(f8.2,f12.4,f12.6,f8.2)
stop
end
